# Script to load the changes in OLR from each
# of the Shell radiative kernels for each model in clear-sky and all-sky,
# make maps of the OLR changes, and calculate global means.
# (Separate script avoids loading the regridded kernels--one less
# dimension, don't have pressure dimension.) 
# 
# Rick Russotto
# Started 1-20-18
#
# 3-28-18: add CESM for water vapor kernel

#import numpy as np
import matplotlib.pyplot as plt
import numpy as np
import netCDF4 as nc4
import geomipFunctions as gmf
import os
from mpl_toolkits.basemap import Basemap
from collections import OrderedDict


#####   CALCULATIONS/LOADING   #####

models_ta = ['bn', 'ca', 'cc', 'ce', 'cs', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']
models_hus = ['bn', 'ca', 'cc', 'ce', 'cs', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']


dLW_ta_G1minusPI = OrderedDict()
dLW_ta_A4minusPI = OrderedDict()                            
dLW_clr_ta_G1minusPI = OrderedDict() 
dLW_clr_ta_A4minusPI = OrderedDict()
dLW_ts_G1minusPI = OrderedDict()
dLW_ts_A4minusPI = OrderedDict()                            
dLW_clr_ts_G1minusPI = OrderedDict() 
dLW_clr_ts_A4minusPI = OrderedDict()
dLW_CO2 = OrderedDict()
dLW_clr_CO2 = OrderedDict()    

dLW_wv_G1minusPI = OrderedDict()
dLW_wv_A4minusPI = OrderedDict()                            
dLW_clr_wv_G1minusPI = OrderedDict() 
dLW_clr_wv_A4minusPI = OrderedDict()

dLW_ta_G1minusPI_annualMean = OrderedDict()
dLW_ta_A4minusPI_annualMean = OrderedDict()                            
dLW_clr_ta_G1minusPI_annualMean = OrderedDict() 
dLW_clr_ta_A4minusPI_annualMean = OrderedDict()
dLW_ts_G1minusPI_annualMean = OrderedDict()
dLW_ts_A4minusPI_annualMean = OrderedDict()                            
dLW_clr_ts_G1minusPI_annualMean = OrderedDict() 
dLW_clr_ts_A4minusPI_annualMean = OrderedDict()
dLW_CO2_annualMean = OrderedDict()
dLW_clr_CO2_annualMean = OrderedDict()

dLW_wv_G1minusPI_annualMean = OrderedDict()
dLW_wv_A4minusPI_annualMean = OrderedDict()                            
dLW_clr_wv_G1minusPI_annualMean = OrderedDict() 
dLW_clr_wv_A4minusPI_annualMean = OrderedDict()

dLW_cloudMasking_G1minusPI_annualMean = OrderedDict()
dLW_cloudMasking_A4minusPI_annualMean = OrderedDict()
    
loadAnnualMean = False; #If true, skip taking annual means and saving, and just load the annual means

#npy files loaded from "shellKernelCalculations.py"
if not loadAnnualMean:
    #Load the OLR changes
    for key in models_ta:
        print('Loading monthly dLW files for model: ' + gmf.modelNames[key])
        dLW_ta_G1minusPI[key] = np.load('npyFiles/shellKernels/dLW_ta_G1minusPI_'+key+'.npy')
        dLW_ta_A4minusPI[key] = np.load('npyFiles/shellKernels/dLW_ta_A4minusPI_'+key+'.npy')
        dLW_clr_ta_G1minusPI[key] = np.load('npyFiles/shellKernels/dLW_clr_ta_G1minusPI_'+key+'.npy')
        dLW_clr_ta_A4minusPI[key] = np.load('npyFiles/shellKernels/dLW_clr_ta_A4minusPI_'+key+'.npy')
        dLW_ts_G1minusPI[key] = np.load('npyFiles/shellKernels/dLW_ts_G1minusPI_'+key+'.npy')
        dLW_ts_A4minusPI[key] = np.load('npyFiles/shellKernels/dLW_ts_A4minusPI_'+key+'.npy')
        dLW_clr_ts_G1minusPI[key] = np.load('npyFiles/shellKernels/dLW_clr_ts_G1minusPI_'+key+'.npy')
        dLW_clr_ts_A4minusPI[key] = np.load('npyFiles/shellKernels/dLW_clr_ts_A4minusPI_'+key+'.npy')
        dLW_CO2[key] = np.load('npyFiles/shellKernels/dLW_CO2_'+key+'.npy')
        dLW_clr_CO2[key] = np.load('npyFiles/shellKernels/dLW_clr_CO2_'+key+'.npy')
    
    for key in models_hus:
        print('Loading monthly dLW files (water vapor) for model: ' + gmf.modelNames[key])
        dLW_wv_G1minusPI[key] = np.load('npyFiles/shellKernels/dLW_wv_G1minusPI_'+key+'.npy')
        dLW_wv_A4minusPI[key] = np.load('npyFiles/shellKernels/dLW_wv_A4minusPI_'+key+'.npy')
        dLW_clr_wv_G1minusPI[key] = np.load('npyFiles/shellKernels/dLW_clr_wv_G1minusPI_'+key+'.npy')
        dLW_clr_wv_A4minusPI[key] = np.load('npyFiles/shellKernels/dLW_clr_wv_A4minusPI_'+key+'.npy')
        
    #Take annual means and save annual mean npy files
    for key in models_ta:
        print('Calculating dLW annual means for model: ' + gmf.modelNames[key])
        dLW_ta_G1minusPI_annualMean[key] = np.mean(dLW_ta_G1minusPI[key], axis=0)
        dLW_ta_A4minusPI_annualMean[key] = np.mean(dLW_ta_A4minusPI[key], axis=0)
        dLW_clr_ta_G1minusPI_annualMean[key] = np.mean(dLW_clr_ta_G1minusPI[key], axis=0)
        dLW_clr_ta_A4minusPI_annualMean[key] = np.mean(dLW_clr_ta_A4minusPI[key], axis=0)
        dLW_ts_G1minusPI_annualMean[key] = np.mean(dLW_ts_G1minusPI[key], axis=0)
        dLW_ts_A4minusPI_annualMean[key] = np.mean(dLW_ts_A4minusPI[key], axis=0)
        dLW_clr_ts_G1minusPI_annualMean[key] = np.mean(dLW_clr_ts_G1minusPI[key], axis=0)
        dLW_clr_ts_A4minusPI_annualMean[key] = np.mean(dLW_clr_ts_A4minusPI[key], axis=0)
        dLW_CO2_annualMean[key] = np.mean(dLW_CO2[key], axis=0)
        dLW_clr_CO2_annualMean[key] = np.mean(dLW_clr_CO2[key], axis=0)
        dLW_ta_G1minusPI_annualMean[key].dump('npyFiles/shellKernels/dLW_ta_G1minusPI_annualMean_'+key+'.npy')    
        dLW_ta_A4minusPI_annualMean[key].dump('npyFiles/shellKernels/dLW_ta_A4minusPI_annualMean_'+key+'.npy')    
        dLW_clr_ta_G1minusPI_annualMean[key].dump('npyFiles/shellKernels/dLW_clr_ta_G1minusPI_annualMean_'+key+'.npy')    
        dLW_clr_ta_A4minusPI_annualMean[key].dump('npyFiles/shellKernels/dLW_clr_ta_A4minusPI_annualMean_'+key+'.npy')    
        dLW_ts_G1minusPI_annualMean[key].dump('npyFiles/shellKernels/dLW_ts_G1minusPI_annualMean_'+key+'.npy')    
        dLW_ts_A4minusPI_annualMean[key].dump('npyFiles/shellKernels/dLW_ts_A4minusPI_annualMean_'+key+'.npy')    
        dLW_clr_ts_G1minusPI_annualMean[key].dump('npyFiles/shellKernels/dLW_clr_ts_G1minusPI_annualMean_'+key+'.npy')    
        dLW_clr_ts_A4minusPI_annualMean[key].dump('npyFiles/shellKernels/dLW_clr_ts_A4minusPI_annualMean_'+key+'.npy')    
        dLW_CO2_annualMean[key].dump('npyFiles/shellKernels/dLW_CO2_annualMean_'+key+'.npy')    
        dLW_clr_CO2_annualMean[key].dump('npyFiles/shellKernels/dLW_clr_CO2_annualMean_'+key+'.npy')    
        
    for key in models_hus:
        print('Calculating dLW annual means (water vapor) for model: ' + gmf.modelNames[key])
        dLW_wv_G1minusPI_annualMean[key] = np.mean(dLW_wv_G1minusPI[key], axis=0)
        dLW_wv_A4minusPI_annualMean[key] = np.mean(dLW_wv_A4minusPI[key], axis=0)
        dLW_clr_wv_G1minusPI_annualMean[key] = np.mean(dLW_clr_wv_G1minusPI[key], axis=0)
        dLW_clr_wv_A4minusPI_annualMean[key] = np.mean(dLW_clr_wv_A4minusPI[key], axis=0)
        dLW_wv_G1minusPI_annualMean[key].dump('npyFiles/shellKernels/dLW_wv_G1minusPI_annualMean_'+key+'.npy')
        dLW_wv_A4minusPI_annualMean[key].dump('npyFiles/shellKernels/dLW_wv_A4minusPI_annualMean_'+key+'.npy')
        dLW_clr_wv_G1minusPI_annualMean[key].dump('npyFiles/shellKernels/dLW_clr_wv_G1minusPI_annualMean_'+key+'.npy')
        dLW_clr_wv_A4minusPI_annualMean[key].dump('npyFiles/shellKernels/dLW_clr_wv_A4minusPI_annualMean_'+key+'.npy')
        
    #Calculate the correction for the LW CRE (annual mean) and save to npy files--to load in "mapLWCRE.py"
    #This is "cloud masking" effects, or the difference between clear-sky and all-sky for the CO2 forcing and 
    #surface temperature, air temperature and water vapor feedbacks. 
    #Can't do this for the model (CESM) where I don't have humidity output.
    #All sky minus clear sky = cloud effect
    for key in models_hus:
        print('Calculating cloud masking effects for model: ' + gmf.modelNames[key])
        dLW_cloudMasking_G1minusPI_annualMean[key] = (dLW_ta_G1minusPI_annualMean[key] - dLW_clr_ta_G1minusPI_annualMean[key] +
                                                      dLW_ts_G1minusPI_annualMean[key] - dLW_clr_ts_G1minusPI_annualMean[key] +
                                                      dLW_wv_G1minusPI_annualMean[key] - dLW_clr_wv_G1minusPI_annualMean[key] +
                                                      dLW_CO2_annualMean[key] - dLW_clr_CO2_annualMean[key])
        dLW_cloudMasking_A4minusPI_annualMean[key] = (dLW_ta_A4minusPI_annualMean[key] - dLW_clr_ta_A4minusPI_annualMean[key] +
                                                      dLW_ts_A4minusPI_annualMean[key] - dLW_clr_ts_A4minusPI_annualMean[key] +
                                                      dLW_wv_A4minusPI_annualMean[key] - dLW_clr_wv_A4minusPI_annualMean[key] +
                                                      dLW_CO2_annualMean[key] - dLW_clr_CO2_annualMean[key])
        dLW_cloudMasking_G1minusPI_annualMean[key].dump('npyFiles/shellKernels_dLW_cloudMasking_G1minusPI_annualMean_'+key+'.npy')
        dLW_cloudMasking_A4minusPI_annualMean[key].dump('npyFiles/shellKernels_dLW_cloudMasking_A4minusPI_annualMean_'+key+'.npy')
                              
else: #Load annual mean npy files
    for key in models_ta:
        print('Loading annual mean kernel OLR changes for: ' + gmf.modelNames[key])
        dLW_ta_G1minusPI_annualMean[key] = np.load('npyFiles/shellKernels/dLW_ta_G1minusPI_annualMean_'+key+'.npy')    
        dLW_ta_A4minusPI_annualMean[key] = np.load('npyFiles/shellKernels/dLW_ta_A4minusPI_annualMean_'+key+'.npy')    
        dLW_clr_ta_G1minusPI_annualMean[key] = np.load('npyFiles/shellKernels/dLW_clr_ta_G1minusPI_annualMean_'+key+'.npy')    
        dLW_clr_ta_A4minusPI_annualMean[key] = np.load('npyFiles/shellKernels/dLW_clr_ta_A4minusPI_annualMean_'+key+'.npy')    
        dLW_ts_G1minusPI_annualMean[key] = np.load('npyFiles/shellKernels/dLW_ts_G1minusPI_annualMean_'+key+'.npy')    
        dLW_ts_A4minusPI_annualMean[key] = np.load('npyFiles/shellKernels/dLW_ts_A4minusPI_annualMean_'+key+'.npy')    
        dLW_clr_ts_G1minusPI_annualMean[key] = np.load('npyFiles/shellKernels/dLW_clr_ts_G1minusPI_annualMean_'+key+'.npy')    
        dLW_clr_ts_A4minusPI_annualMean[key] = np.load('npyFiles/shellKernels/dLW_clr_ts_A4minusPI_annualMean_'+key+'.npy')    
        dLW_CO2_annualMean[key] = np.load('npyFiles/shellKernels/dLW_CO2_annualMean_'+key+'.npy')    
        dLW_clr_CO2_annualMean[key] = np.load('npyFiles/shellKernels/dLW_clr_CO2_annualMean_'+key+'.npy')    
        
    for key in models_hus:
        print('Loading annual mean kernel OLR changes (water vapor and cloud masking) for: ' + gmf.modelNames[key])
        dLW_wv_G1minusPI_annualMean[key] = np.load('npyFiles/shellKernels/dLW_wv_G1minusPI_annualMean_'+key+'.npy')
        dLW_wv_A4minusPI_annualMean[key] = np.load('npyFiles/shellKernels/dLW_wv_A4minusPI_annualMean_'+key+'.npy')
        dLW_clr_wv_G1minusPI_annualMean[key] = np.load('npyFiles/shellKernels/dLW_clr_wv_G1minusPI_annualMean_'+key+'.npy')
        dLW_clr_wv_A4minusPI_annualMean[key] = np.load('npyFiles/shellKernels/dLW_clr_wv_A4minusPI_annualMean_'+key+'.npy')
        dLW_cloudMasking_G1minusPI_annualMean[key] = np.load('npyFiles/shellKernels_dLW_cloudMasking_G1minusPI_annualMean_'+key+'.npy')
        dLW_cloudMasking_A4minusPI_annualMean[key] = np.load('npyFiles/shellKernels_dLW_cloudMasking_A4minusPI_annualMean_'+key+'.npy')
        
        
#####   SINGLE-MODEL PLOTS   #####        
        
#Load lat and lon for each model
modelLats = np.load('npyFiles/modelLats_ta.npy').item()
modelLons = np.load('npyFiles/modelLons_ta.npy').item()
        


#PNG versions of individual plots, with bigger and simpler text, for paper supplement
def mapDiffContourfPNG(figNo, key, run1, run2, plotDict, clevs='default', titleString='', saveString = '', save=False):
    w,h = plt.figaspect(1./2.5)  
    plt.figure(figNo, figsize=(w,h))    
    lat = modelLats[key]
    lon = modelLons[key]
    plotData = plotDict[key]
    titleString = gmf.modelNames[key]
    #Duplicate edge data to eliminate white strip on left side
    lon = np.append(lon,360.) 
    plotData = np.append(plotData, plotData[:,0:1], axis=1)  
    #Make map
    Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
    Map.drawcoastlines(linewidth=1)
    Map.drawparallels(np.linspace(-90.,90.,7))
    Map.drawmeridians(np.linspace(-150., 150., 11))
    lons, lats = np.meshgrid(lon, lat)
    if clevs == 'default':
        thePlot = Map.contourf(lons, lats, plotData, latlon=True, cmap=plt.cm.RdBu)
    else: 
        thePlot = Map.contourf(lons, lats, plotData, latlon=True, cmap=plt.cm.RdBu, levels=clevs, extend='both')
    cbar = plt.colorbar(thePlot)
    cbar.ax.set_ylabel('W m$^{-2}$', fontsize=24)    
    cbar.ax.tick_params(labelsize=22)
    tickLocs = np.linspace(min(clevs), max(clevs), len(clevs)-1)
    cbar.set_ticks(tickLocs)
    cbar.set_ticklabels(np.around(tickLocs, 3))
    plt.title(titleString, fontsize=26)
    plt.tight_layout()
    if(save):   
        plt.savefig('plots/individualModels/'+saveString+'_map_'+run1+'-'+run2+'_'+key+'.png') 

ts1 = '$\Delta$OLR from air temperature kernel'
ss1 = 'dLW_ta_G1minusPI'
mapDiffContourfPNG(800,  'bn', 'G1', 'PI', dLW_ta_G1minusPI_annualMean, titleString = ts1, clevs = np.linspace(-8,8,12), saveString=ss1, save=True)
mapDiffContourfPNG(801,  'ca', 'G1', 'PI', dLW_ta_G1minusPI_annualMean, titleString = ts1, clevs = np.linspace(-8,8,12), saveString=ss1, save=True)
mapDiffContourfPNG(802,  'cc', 'G1', 'PI', dLW_ta_G1minusPI_annualMean, titleString = ts1, clevs = np.linspace(-8,8,12), saveString=ss1, save=True)
mapDiffContourfPNG(803,  'ce', 'G1', 'PI', dLW_ta_G1minusPI_annualMean, titleString = ts1, clevs = np.linspace(-8,8,12), saveString=ss1, save=True)
mapDiffContourfPNG(804,  'cs', 'G1', 'PI', dLW_ta_G1minusPI_annualMean, titleString = ts1, clevs = np.linspace(-8,8,12), saveString=ss1, save=True)
mapDiffContourfPNG(805,  'gi', 'G1', 'PI', dLW_ta_G1minusPI_annualMean, titleString = ts1, clevs = np.linspace(-8,8,12), saveString=ss1, save=True)
mapDiffContourfPNG(806,  'hg', 'G1', 'PI', dLW_ta_G1minusPI_annualMean, titleString = ts1, clevs = np.linspace(-8,8,12), saveString=ss1, save=True)
mapDiffContourfPNG(807,  'ip', 'G1', 'PI', dLW_ta_G1minusPI_annualMean, titleString = ts1, clevs = np.linspace(-8,8,12), saveString=ss1, save=True)
mapDiffContourfPNG(808,  'mi', 'G1', 'PI', dLW_ta_G1minusPI_annualMean, titleString = ts1, clevs = np.linspace(-8,8,12), saveString=ss1, save=True)
mapDiffContourfPNG(809,  'mp', 'G1', 'PI', dLW_ta_G1minusPI_annualMean, titleString = ts1, clevs = np.linspace(-8,8,12), saveString=ss1, save=True)
mapDiffContourfPNG(810,  'no', 'G1', 'PI', dLW_ta_G1minusPI_annualMean, titleString = ts1, clevs = np.linspace(-8,8,12), saveString=ss1, save=True)

ts3 = '$\Delta$OLR from surface temperature kernel'
ss3 = 'dLW_ts_G1minusPI'
mapDiffContourfPNG(840,  'bn', 'G1', 'PI', dLW_ts_G1minusPI_annualMean, titleString = ts3, clevs = np.linspace(-5,5,12), saveString=ss3, save=True)
mapDiffContourfPNG(841,  'ca', 'G1', 'PI', dLW_ts_G1minusPI_annualMean, titleString = ts3, clevs = np.linspace(-5,5,12), saveString=ss3, save=True)
mapDiffContourfPNG(842,  'cc', 'G1', 'PI', dLW_ts_G1minusPI_annualMean, titleString = ts3, clevs = np.linspace(-5,5,12), saveString=ss3, save=True)
mapDiffContourfPNG(843,  'ce', 'G1', 'PI', dLW_ts_G1minusPI_annualMean, titleString = ts3, clevs = np.linspace(-5,5,12), saveString=ss3, save=True)
mapDiffContourfPNG(844,  'cs', 'G1', 'PI', dLW_ts_G1minusPI_annualMean, titleString = ts3, clevs = np.linspace(-5,5,12), saveString=ss3, save=True)
mapDiffContourfPNG(845,  'gi', 'G1', 'PI', dLW_ts_G1minusPI_annualMean, titleString = ts3, clevs = np.linspace(-5,5,12), saveString=ss3, save=True)
mapDiffContourfPNG(846,  'hg', 'G1', 'PI', dLW_ts_G1minusPI_annualMean, titleString = ts3, clevs = np.linspace(-5,5,12), saveString=ss3, save=True)
mapDiffContourfPNG(847,  'ip', 'G1', 'PI', dLW_ts_G1minusPI_annualMean, titleString = ts3, clevs = np.linspace(-5,5,12), saveString=ss3, save=True)
mapDiffContourfPNG(848,  'mi', 'G1', 'PI', dLW_ts_G1minusPI_annualMean, titleString = ts3, clevs = np.linspace(-5,5,12), saveString=ss3, save=True)
mapDiffContourfPNG(849,  'mp', 'G1', 'PI', dLW_ts_G1minusPI_annualMean, titleString = ts3, clevs = np.linspace(-5,5,12), saveString=ss3, save=True)
mapDiffContourfPNG(850,  'no', 'G1', 'PI', dLW_ts_G1minusPI_annualMean, titleString = ts3, clevs = np.linspace(-5,5,12), saveString=ss3, save=True)

ts5 = '$\Delta$OLR from water vapor kernel'
ss5 = 'dLW_wv_G1minusPI'
mapDiffContourfPNG(880,  'bn', 'G1', 'PI', dLW_wv_G1minusPI_annualMean, titleString = ts5, clevs = np.linspace(-5,5,12), saveString=ss5, save=True)
mapDiffContourfPNG(881,  'ca', 'G1', 'PI', dLW_wv_G1minusPI_annualMean, titleString = ts5, clevs = np.linspace(-5,5,12), saveString=ss5, save=True)
mapDiffContourfPNG(882,  'cc', 'G1', 'PI', dLW_wv_G1minusPI_annualMean, titleString = ts5, clevs = np.linspace(-5,5,12), saveString=ss5, save=True)
mapDiffContourfPNG(882,  'ce', 'G1', 'PI', dLW_wv_G1minusPI_annualMean, titleString = ts5, clevs = np.linspace(-5,5,12), saveString=ss5, save=True)
mapDiffContourfPNG(883,  'cs', 'G1', 'PI', dLW_wv_G1minusPI_annualMean, titleString = ts5, clevs = np.linspace(-5,5,12), saveString=ss5, save=True)
mapDiffContourfPNG(884,  'gi', 'G1', 'PI', dLW_wv_G1minusPI_annualMean, titleString = ts5, clevs = np.linspace(-5,5,12), saveString=ss5, save=True)
mapDiffContourfPNG(885,  'hg', 'G1', 'PI', dLW_wv_G1minusPI_annualMean, titleString = ts5, clevs = np.linspace(-5,5,12), saveString=ss5, save=True)
mapDiffContourfPNG(886,  'ip', 'G1', 'PI', dLW_wv_G1minusPI_annualMean, titleString = ts5, clevs = np.linspace(-5,5,12), saveString=ss5, save=True)
mapDiffContourfPNG(887,  'mi', 'G1', 'PI', dLW_wv_G1minusPI_annualMean, titleString = ts5, clevs = np.linspace(-5,5,12), saveString=ss5, save=True)
mapDiffContourfPNG(888,  'mp', 'G1', 'PI', dLW_wv_G1minusPI_annualMean, titleString = ts5, clevs = np.linspace(-5,5,12), saveString=ss5, save=True)
mapDiffContourfPNG(889,  'no', 'G1', 'PI', dLW_wv_G1minusPI_annualMean, titleString = ts5, clevs = np.linspace(-5,5,12), saveString=ss5, save=True)



#####   MULTI-MODEL MEAN PLOTS   #####

#Make multi-model mean maps
#For regridding, go from -86 to 86 latitude--to avoid issue of losing the mask where polar values have been masked out. 


#3-28-18: add CESM for individual model plots first; then redo multi-model mean. 
skipMMM = False
if not skipMMM:
    
    #Lats and lons to interpolate to (1-degree grid)
    interpLats = np.linspace(-86., 86., 173)
    interpLons = np.linspace(0., 360., 361)
    w,h = plt.figaspect(1./2.5)  
    
    #Regrid and make multi-model means
    #Hatched where less than all but 2 of models agree on sign of change 
    print('Multi-model-mean, ' + ts1 + ':')
    collocated_dLW_ta_G1minusPI = gmf.collocateModels(dLW_ta_G1minusPI_annualMean, modelLats, modelLons, interpLats, interpLons, method='linear')
    mmm_dLW_ta_G1minusPI = np.mean(collocated_dLW_ta_G1minusPI, axis=2)
    agreeBut2_dLW_ta_G1minusPI = gmf.arraySignAgreement3D(collocated_dLW_ta_G1minusPI, 2)
    
    
    #Big text version for paper
    plt.figure(801, figsize=(w,h))
    Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
    Map.drawcoastlines(linewidth=1)
    Map.drawparallels(np.linspace(-90.,90.,7))
    Map.drawmeridians(np.linspace(-150., 150., 11))
    lon = interpLons
    lat = interpLats    
    lons, lats = np.meshgrid(lon, lat)
    plotData = mmm_dLW_ta_G1minusPI
    cmap = plt.cm.RdBu                            
    clevs=np.linspace(-5., 5., 12)                
    Map.contourf(lons, lats, plotData, latlon=True, cmap=plt.cm.RdBu, levels=clevs, extend='both') 
    cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
    cbar.ax.set_ylabel('$\Delta$ OLR (W m$^{-2})$', fontsize=24)
    cbar.ax.tick_params(labelsize=22)   
    plt.title('(a) LW radiative effect of air temperature changes', fontsize=26)
    #Add hatching (next 2 lines)
    zm = np.ma.masked_where(agreeBut2_dLW_ta_G1minusPI == True, plotData)
    Map.pcolor(lons, lats, zm, latlon=True, cmap=plt.cm.RdBu_r, alpha=0, hatch='xx') 
    plt.tight_layout()
    #plt.savefig('plots/multiModelMean_dLW_ta_G1minusPI_bigText.png')
    
    
    
    
    
    
    
    print('Multi-model-mean, ' + ts3 + ':')
    collocated_dLW_ts_G1minusPI = gmf.collocateModels(dLW_ts_G1minusPI_annualMean, modelLats, modelLons, interpLats, interpLons, method='linear')
    mmm_dLW_ts_G1minusPI = np.mean(collocated_dLW_ts_G1minusPI, axis=2)
    agreeBut2_dLW_ts_G1minusPI = gmf.arraySignAgreement3D(collocated_dLW_ts_G1minusPI, 2)
    
    
    
    #Bigger text version for paper
    plt.figure(803, figsize=(w,h))
    Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
    Map.drawcoastlines(linewidth=1)
    Map.drawparallels(np.linspace(-90.,90.,7))
    Map.drawmeridians(np.linspace(-150., 150., 11))
    lon = interpLons
    lat = interpLats    
    lons, lats = np.meshgrid(lon, lat)
    plotData = mmm_dLW_ts_G1minusPI
    cmap = plt.cm.RdBu                            
    clevs=np.linspace(-5., 5., 12)                
    Map.contourf(lons, lats, plotData, latlon=True, cmap=plt.cm.RdBu, levels=clevs, extend='both') 
    cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
    cbar.ax.set_ylabel('$\Delta$ OLR (W m$^{-2})$', fontsize=24)
    cbar.ax.tick_params(labelsize=22)   
    plt.title('(b) LW radiative effect of surface temperature changes', fontsize=26)
    #Add hatching (next 2 lines)
    zm = np.ma.masked_where(agreeBut2_dLW_ts_G1minusPI == True, plotData)
    Map.pcolor(lons, lats, zm, latlon=True, cmap=plt.cm.RdBu_r, alpha=0, hatch='xx') 
    plt.tight_layout()
    #plt.savefig('plots/multiModelMean_dLW_ts_G1minusPI_bigText.png')
    
    
    
    
    
    
    
    print('Multi-model-mean, ' + ts5 + ':')
    collocated_dLW_wv_G1minusPI = gmf.collocateModels(dLW_wv_G1minusPI_annualMean, modelLats, modelLons, interpLats, interpLons, method='linear')
    mmm_dLW_wv_G1minusPI = np.mean(collocated_dLW_wv_G1minusPI, axis=2)
    agreeBut2_dLW_wv_G1minusPI = gmf.arraySignAgreement3D(collocated_dLW_wv_G1minusPI, 2)
    
    
    #Bigger text version for paper
    plt.figure(805, figsize=(w,h))
    Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
    Map.drawcoastlines(linewidth=1)
    Map.drawparallels(np.linspace(-90.,90.,7))
    Map.drawmeridians(np.linspace(-150., 150., 11))
    lon = interpLons
    lat = interpLats    
    lons, lats = np.meshgrid(lon, lat)
    plotData = mmm_dLW_wv_G1minusPI
    cmap = plt.cm.RdBu                            
    clevs=np.linspace(-5., 5., 12)                
    Map.contourf(lons, lats, plotData, latlon=True, cmap=plt.cm.RdBu, levels=clevs, extend='both') 
    cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
    cbar.ax.set_ylabel('$\Delta$ OLR (W m$^{-2})$', fontsize=24)
    cbar.ax.tick_params(labelsize=22)   
    plt.title('(c) LW radiative effect of water vapor changes', fontsize=26)
    #Add hatching (next 2 lines)
    zm = np.ma.masked_where(agreeBut2_dLW_wv_G1minusPI == True, plotData)
    Map.pcolor(lons, lats, zm, latlon=True, cmap=plt.cm.RdBu_r, alpha=0, hatch='xx') 
    plt.tight_layout()
    plt.savefig('plots/multiModelMean_dLW_wv_G1minusPI_bigText.png')
    
    
    
    plt.show()



#####   CALCULATE GLOBAL MEANS OF ALL THE OLR CHANGES   #####

#Save to npy files and put in console output--copy into here. 

#Possible issue with this: will it preserve masks? 
#Well, "Data" is just multplied by another 2D matrix and then summed. So it will.
#However, the area weights should be masked as well.


def calcGlobalMean(dictName, key):
    Data = dictName[key]
    Data = np.ma.masked_where(np.isnan(Data), Data) #Default outside "convex hull" for regridding was nan--convert this to masked array
                                                    #This statement may make the "hasattr" check below unnecesary. 
    lat = modelLats[key]
    areaWeights = np.cos(lat*np.pi/180)
    areaWeights2D = np.tile(areaWeights,(np.shape(Data)[1],1))
    #print(np.shape(areaWeights2D)) #debug
    areaWeights2D = np.swapaxes(areaWeights2D,0,1)
    if hasattr(Data, 'mask'): #Need to give the area weights the same mask as the data to avoid bias when dividing by sum of weights
        areaWeights2D_mask = np.ma.masked_where(Data.mask == True, areaWeights2D)
        weighted2DMatrix = Data*areaWeights2D_mask
        sumWeighted = np.sum(weighted2DMatrix)
        sumWeights = np.sum(areaWeights2D_mask)
    else:  #don't mask the weights (data not masked array--attempting to access mask will cause error)
        weighted2DMatrix = Data*areaWeights2D
        sumWeighted = np.sum(weighted2DMatrix)
        sumWeights = np.sum(areaWeights2D)
    weightedMean = sumWeighted/sumWeights
    return weightedMean
    
skipGlobalMeans = False

if not skipGlobalMeans:
    
    globalMean_dLW_ta_G1minusPI = OrderedDict()
    globalMean_dLW_ta_A4minusPI = OrderedDict()                            
    globalMean_dLW_clr_ta_G1minusPI = OrderedDict() 
    globalMean_dLW_clr_ta_A4minusPI = OrderedDict()
    globalMean_dLW_ts_G1minusPI = OrderedDict()
    globalMean_dLW_ts_A4minusPI = OrderedDict()                            
    globalMean_dLW_clr_ts_G1minusPI = OrderedDict() 
    globalMean_dLW_clr_ts_A4minusPI = OrderedDict()
    globalMean_dLW_CO2 = OrderedDict() #Differences between these can test effects of resolution
    globalMean_dLW_clr_CO2 = OrderedDict()
    
    globalMean_dLW_wv_G1minusPI = OrderedDict()
    globalMean_dLW_wv_A4minusPI = OrderedDict()                            
    globalMean_dLW_clr_wv_G1minusPI = OrderedDict() 
    globalMean_dLW_clr_wv_A4minusPI = OrderedDict()
    globalMean_dLW_cloudMasking_G1minusPI = OrderedDict()
    globalMean_dLW_cloudMasking_A4minusPI = OrderedDict()
    
    for key in models_ta:
        print(key)
        print(dLW_ta_G1minusPI_annualMean[key])
        print(dLW_ts_G1minusPI_annualMean[key])
        globalMean_dLW_ta_G1minusPI[key] = calcGlobalMean(dLW_ta_G1minusPI_annualMean, key)
        globalMean_dLW_ta_A4minusPI[key] = calcGlobalMean(dLW_ta_A4minusPI_annualMean, key)
        globalMean_dLW_clr_ta_G1minusPI[key] = calcGlobalMean(dLW_clr_ta_G1minusPI_annualMean, key)
        globalMean_dLW_clr_ta_A4minusPI[key] = calcGlobalMean(dLW_clr_ta_A4minusPI_annualMean, key)
        globalMean_dLW_ts_G1minusPI[key] = calcGlobalMean(dLW_ts_G1minusPI_annualMean, key)
        globalMean_dLW_ts_A4minusPI[key] = calcGlobalMean(dLW_ts_A4minusPI_annualMean, key)                            
        globalMean_dLW_clr_ts_G1minusPI[key] = calcGlobalMean(dLW_clr_ts_G1minusPI_annualMean, key) 
        globalMean_dLW_clr_ts_A4minusPI[key] = calcGlobalMean(dLW_clr_ts_A4minusPI_annualMean, key)
        globalMean_dLW_CO2[key] = calcGlobalMean(dLW_CO2_annualMean, key)
        globalMean_dLW_clr_CO2[key] = calcGlobalMean(dLW_clr_CO2_annualMean, key)
    
        
    for key in models_hus:
        globalMean_dLW_wv_G1minusPI[key] = calcGlobalMean(dLW_wv_G1minusPI_annualMean, key)
        globalMean_dLW_wv_A4minusPI[key] = calcGlobalMean(dLW_wv_A4minusPI_annualMean, key)
        globalMean_dLW_clr_wv_G1minusPI[key] = calcGlobalMean(dLW_clr_wv_G1minusPI_annualMean, key)
        globalMean_dLW_clr_wv_A4minusPI[key] = calcGlobalMean(dLW_clr_wv_A4minusPI_annualMean, key)
        globalMean_dLW_cloudMasking_G1minusPI[key] = calcGlobalMean(dLW_cloudMasking_G1minusPI_annualMean, key)
        globalMean_dLW_cloudMasking_A4minusPI[key] = calcGlobalMean(dLW_cloudMasking_A4minusPI_annualMean, key)
        
        
    #Save npy files    
    np.save('npyFiles/shellKernels/globalMean_dLW_ta_G1minusPI.npy', globalMean_dLW_ta_G1minusPI)    
    np.save('npyFiles/shellKernels/globalMean_dLW_ta_A4minusPI.npy', globalMean_dLW_ta_A4minusPI)    
    np.save('npyFiles/shellKernels/globalMean_dLW_clr_ta_G1minusPI.npy', globalMean_dLW_clr_ta_G1minusPI)    
    np.save('npyFiles/shellKernels/globalMean_dLW_clr_ta_A4minusPI.npy', globalMean_dLW_clr_ta_A4minusPI)    
    
    np.save('npyFiles/shellKernels/globalMean_dLW_ts_G1minusPI.npy', globalMean_dLW_ts_G1minusPI)    
    np.save('npyFiles/shellKernels/globalMean_dLW_ts_A4minusPI.npy', globalMean_dLW_ts_A4minusPI)    
    np.save('npyFiles/shellKernels/globalMean_dLW_clr_ts_G1minusPI.npy', globalMean_dLW_clr_ts_G1minusPI)    
    np.save('npyFiles/shellKernels/globalMean_dLW_clr_ts_A4minusPI.npy', globalMean_dLW_clr_ts_A4minusPI)    
    
    np.save('npyFiles/shellKernels/globalMean_dLW_wv_G1minusPI.npy', globalMean_dLW_wv_G1minusPI)    
    np.save('npyFiles/shellKernels/globalMean_dLW_wv_A4minusPI.npy', globalMean_dLW_wv_A4minusPI)    
    np.save('npyFiles/shellKernels/globalMean_dLW_clr_wv_G1minusPI.npy', globalMean_dLW_clr_wv_G1minusPI)    
    np.save('npyFiles/shellKernels/globalMean_dLW_clr_wv_A4minusPI.npy', globalMean_dLW_clr_wv_A4minusPI)    
    
    np.save('npyFiles/shellKernels/globalMean_dLW_CO2.npy', globalMean_dLW_CO2)
    np.save('npyFiles/shellKernels/globalMean_dLW_clr_CO2.npy', globalMean_dLW_clr_CO2)
    
    np.save('npyFiles/shellKernels/globalMean_dLW_cloudMasking_G1minusPI', globalMean_dLW_cloudMasking_G1minusPI)
    np.save('npyFiles/shellKernels/globalMean_dLW_cloudMasking_A4minusPI', globalMean_dLW_cloudMasking_A4minusPI)
    
    
    #Print console output
    print('Global mean changes in OLR based on Shell radiative kernels:')
    
    print('Global mean dLW_ta_G1minusPI (W/m^2): ')
    for key in models_ta:
        print(key + ': ' + str(globalMean_dLW_ta_G1minusPI[key]))
        
    
    print('Global mean dLW_clr_ta_G1minusPI (W/m^2): ')
    for key in models_ta:
        print(key + ': ' + str(globalMean_dLW_clr_ta_G1minusPI[key]))
        
    
    print('Global mean dLW_ts_G1minusPI (W/m^2): ')
    for key in models_ta:
        print(key + ': ' + str(globalMean_dLW_ts_G1minusPI[key]))
    
    
    print('Global mean dLW_clr_ts_G1minusPI (W/m^2): ')
    for key in models_ta:
        print(key + ': ' + str(globalMean_dLW_clr_ts_G1minusPI[key]))
    
    print('Global mean dLW_wv_G1minusPI (W/m^2): ')
    for key in models_hus:
        print(key + ': ' + str(globalMean_dLW_wv_G1minusPI[key]))
    
    
    print('Global mean dLW_clr_wv_G1minusPI (W/m^2): ')
    for key in models_hus:
        print(key + ': ' + str(globalMean_dLW_clr_wv_G1minusPI[key]))    
        
        
        
        
        
    print('Global mean dLW_ta_A4minusPI (W/m^2): ')
    for key in models_ta:
        print(key + ': ' + str(globalMean_dLW_ta_A4minusPI[key]))
        
    
    print('Global mean dLW_clr_ta_A4minusPI (W/m^2): ')
    for key in models_ta:
        print(key + ': ' + str(globalMean_dLW_clr_ta_A4minusPI[key]))
        
    
    print('Global mean dLW_ts_A4minusPI (W/m^2): ')
    for key in models_ta:
        print(key + ': ' + str(globalMean_dLW_ts_A4minusPI[key]))
    
    
    print('Global mean dLW_clr_ts_A4minusPI (W/m^2): ')
    for key in models_ta:
        print(key + ': ' + str(globalMean_dLW_clr_ts_A4minusPI[key]))
    
    print('Global mean dLW_wv_A4minusPI (W/m^2): ')
    for key in models_hus:
        print(key + ': ' + str(globalMean_dLW_wv_A4minusPI[key]))
    
    
    print('Global mean dLW_clr_wv_A4minusPI (W/m^2): ')
    for key in models_hus:
        print(key + ': ' + str(globalMean_dLW_clr_wv_A4minusPI[key]))        
        
        
        
    print('Global mean dLW_CO2 (W/m^2): ')
    for key in models_ta:
        print(key + ': ' + str(globalMean_dLW_CO2[key]))        
        
    print('Global mean dLW_clr_CO2 (W/m^2): ')
    for key in models_ta:
        print(key + ': ' + str(globalMean_dLW_clr_CO2[key]))        
        
    print('Global mean dLW_cloudMasking_G1minusPI (W/m^2): ')
    for key in models_hus:
        print(key + ': ' + str(globalMean_dLW_cloudMasking_G1minusPI[key]))        
        
    print('Global mean dLW_cloudMasking_A4minusPI (W/m^2): ')
    for key in models_hus:
        print(key + ': ' + str(globalMean_dLW_cloudMasking_A4minusPI[key]))        
        
    
    
#Console output from all the printing above: (some of these values are listed in Table 3 of paper)
#Global mean changes in OLR based on Shell radiative kernels:
#Global mean dLW_ta_G1minusPI (W/m^2): 
#bn: -2.94124174338
#ca: -3.03092138742
#cc: -2.52515261947
#ce: -3.93586299081
#cs: -2.16358582469
#gi: -4.8819283143
#hg: -2.65711971584
#ip: -2.08456239855
#mi: -3.44472535814
#mp: -3.41357236522
#no: -2.88017845235
#Global mean dLW_clr_ta_G1minusPI (W/m^2): 
#bn: -2.88652998642
#ca: -2.92179312258
#cc: -2.43850191873
#ce: -3.72389327526
#cs: -2.15568824126
#gi: -4.62139601928
#hg: -2.5602422297
#ip: -2.07467964829
#mi: -3.30244657408
#mp: -3.26264128232
#no: -2.74152354166
#Global mean dLW_ts_G1minusPI (W/m^2): 
#bn: -0.0848445827993
#ca: -0.0662226729352
#cc: 0.0788816480597
#ce: -0.177368637109
#cs: -0.0326808297278
#gi: -0.211682361575
#hg: 0.0469967265003
#ip: 0.0468136143011
#mi: -0.0985015311451
#mp: -0.0650909254833
#no: -0.0837273990993
#Global mean dLW_clr_ts_G1minusPI (W/m^2): 
#bn: 0.0734625086314
#ca: -0.00685728600648
#cc: 0.325637412738
#ce: -0.193475771174
#cs: 0.0444310152869
#gi: -0.301289824535
#hg: 0.234619522853
#ip: 0.155088854826
#mi: -0.0860334035012
#mp: -0.0118241816004
#no: -0.0729892923422
#Global mean dLW_wv_G1minusPI (W/m^2): 
#bn: 0.783727287427
#ca: 1.04161634348
#cc: 0.843551953362
#ce: 1.38877319315
#cs: 0.524483015877
#gi: 1.78238643174
#hg: 0.866161886349
#ip: 0.519699068553
#mi: 1.11003246626
#mp: 1.0979893454
#no: 0.96610039243
#Global mean dLW_clr_wv_G1minusPI (W/m^2): 
#bn: 1.04810312107
#ca: 1.46777116872
#cc: 1.10837842925
#ce: 1.88604356974
#cs: 0.726336320789
#gi: 2.47006887929
#hg: 1.11878488705
#ip: 0.70503845721
#mi: 1.52553353882
#mp: 1.49626671589
#no: 1.29617065745
#Global mean dLW_ta_A4minusPI (W/m^2): 
#bn: 12.3037212532
#ca: 12.8200671379
#cc: 9.04758059676
#ce: 10.94279868
#cs: 8.86093362761
#gi: 7.17357786047
#hg: 12.7018925085
#ip: 13.4741576607
#mi: 14.8411447502
#mp: 13.794875848
#no: 7.90504341362
#Global mean dLW_clr_ta_A4minusPI (W/m^2): 
#bn: 10.6977694715
#ca: 11.333754051
#cc: 7.93556774133
#ce: 9.59435802303
#cs: 7.79654987662
#gi: 6.3778694942
#hg: 11.2171050698
#ip: 11.9879838798
#mi: 13.0917035085
#mp: 12.3066670349
#no: 6.97071063286
#Global mean dLW_ts_A4minusPI (W/m^2): 
#bn: 3.33763447073
#ca: 3.13118510858
#cc: 2.48742872284
#ce: 2.93461399114
#cs: 2.26335843309
#gi: 1.97279424721
#hg: 3.2815304653
#ip: 3.05251036206
#mi: 3.61089218392
#mp: 3.12214284356
#no: 2.15975429167
#Global mean dLW_clr_ts_A4minusPI (W/m^2): 
#bn: 6.91810385174
#ca: 6.16057323245
#cc: 5.01958248666
#ce: 5.89258249255
#cs: 4.41477164943
#gi: 3.67915375409
#hg: 6.53866148467
#ip: 5.78226917847
#mi: 7.12200748543
#mp: 5.91147947649
#no: 4.24201498366
#Global mean dLW_wv_A4minusPI (W/m^2): 
#bn: -6.94933480231
#ca: -7.25546765004
#cc: -5.06344948071
#ce: -6.20061651914
#cs: -5.01347906637
#gi: -4.75295588013
#hg: -7.19053294413
#ip: -7.73459667448
#mi: -8.3230518628
#mp: -7.81736827128
#no: -4.46856119025
#Global mean dLW_clr_wv_A4minusPI (W/m^2): 
#bn: -9.55532809101
#ca: -9.8516608446
#cc: -6.99403682234
#ce: -8.45602251202
#cs: -6.8180594767
#gi: -6.45177400903
#hg: -9.89344725115
#ip: -10.6192823852
#mi: -11.3810150738
#mp: -10.7107775645
#no: -6.17587540886
#Global mean dLW_CO2 (W/m^2): 
#bn: nan
#ca: nan
#cc: nan
#ce: nan
#cs: -5.02165053179
#gi: nan
#hg: nan
#ip: nan
#mi: -5.02062084533
#mp: nan
#no: nan
#Global mean dLW_clr_CO2 (W/m^2): 
#bn: nan
#ca: nan
#cc: nan
#ce: nan
#cs: -6.39137882641
#gi: nan
#hg: nan
#ip: nan
#mi: -6.39180044689
#mp: nan
#no: nan
#Global mean dLW_cloudMasking_G1minusPI (W/m^2): 
#bn: 0.895509399142
#ca: 0.778255595289
#cc: 0.773689961576
#ce: 0.678995121377
#cs: 1.08286556126
#gi: 0.513218998033
#hg: 0.835008641841
#ip: 1.068973476
#mi: 0.800931617293
#mp: 0.769282485774
#no: 0.892664805275
#Global mean dLW_cloudMasking_A4minusPI (W/m^2): 
#bn: 2.00437977054
#ca: 2.42602222992
#cc: 1.88236933611
#ce: 2.0180062279
#cs: 2.08727923959
#gi: 2.15999326593
#hg: 2.30270265139
#ip: 3.01357153071
#mi: 2.6674687528
#mp: 2.9640391564
#no: 1.93151439512

