# Script to calculate 40-year averages of ta and hus (temperature and specific humidity) 
# in each month, pressure level, latitude and longitude for multiplying by radiative kernels. 
#
# Do this in individual runs as well as take differences. .
# 
# All this script will do is take averages and save npy files--no plots. 
#
# Rick Russotto
# Started 8 January 2018
#
# 19 January: adding ts--for surface (skin) temperature kernel
# 28 March: adding humidity for CESM

import numpy as np
import netCDF4 as nc4
import geomipFunctions as gmf
from collections import OrderedDict


#Paths for the NetCDF files

#Paths for the files
folder = '../nobackup/' 

#Temperatures
fpaths_ta_PI = OrderedDict()
fpaths_ta_PI['bn'] = folder+'ta_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc'
fpaths_ta_PI['ca'] = folder+'ta_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc'
fpaths_ta_PI['cc'] = folder+'ta_Amon_CCSM4_piControl_r1i1p1_080001-084912.nc'
fpaths_ta_PI['ce'] = folder+'ta_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc'
fpaths_ta_PI['cs'] = folder+'ta_Amon_CSIRO-Mk3L-1-2_piControl_r1i1p1_000101-010012.nc'
fpaths_ta_PI['gi'] = folder+'ta_Amon_GISS-E2-R_piControl_r1i1p1_185001-191912.nc'
fpaths_ta_PI['hg'] = folder+'ta_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc'
fpaths_ta_PI['ip'] = folder+'ta_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-189912.nc'
fpaths_ta_PI['mi'] = folder+'ta_Amon_MIROC-ESM_piControl_r1i1p1_180001-193912.nc'
fpaths_ta_PI['mp'] = folder+'ta_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-189912.nc'
fpaths_ta_PI['no'] = folder+'ta_Amon_NorESM1-M_piControl_r1i1p1_070101-079912.nc'

fpaths_ta_G1 = OrderedDict()
fpaths_ta_G1['bn'] = folder+'ta_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc' 
fpaths_ta_G1['ca'] = folder+'ta_Amon_CanESM2_G1_r1i1p1_185001-194912.nc'
fpaths_ta_G1['cc'] = folder+'ta_Amon_CCSM4_G1_r1i1p1_185001-192412.nc'   
fpaths_ta_G1['ce'] = folder+'ta_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc'
fpaths_ta_G1['cs'] = folder+'ta_Amon_CSIRO-Mk3L-1-2_G1_r1i1p1_000101-007012.nc'
fpaths_ta_G1['gi'] = folder+'ta_Amon_GISS-E2-R_G1_r1i1p1_185001-191912.nc'
fpaths_ta_G1['hg'] = folder+'ta_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc' #Skip for now
fpaths_ta_G1['ip'] = folder+'ta_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc'
fpaths_ta_G1['mi'] = folder+'ta_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc'
fpaths_ta_G1['mp'] = folder+'ta_Amon_MPI-ESM-LR_G1_r2i1p1_185001-191912.nc'
fpaths_ta_G1['no'] = folder+'ta_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc'

fpaths_ta_A4 = OrderedDict()
fpaths_ta_A4['bn'] = folder+'ta_Amon_BNU-ESM_abrupt4xCO2_r1i1p1_185001-199912.nc'
fpaths_ta_A4['ca'] = folder+'ta_Amon_CanESM2_abrupt4xCO2_r1i1p1_185001-199912.nc'
fpaths_ta_A4['cc'] = folder+'ta_Amon_CCSM4_abrupt4xCO2_r1i1p1_185001-189912.nc'
fpaths_ta_A4['ce'] = folder+'ta_Amon_CESM1-CAM5-1-FV2_abrupt4xCO2_r3i1p1_000101-015012.nc'
fpaths_ta_A4['cs'] = folder+'ta_Amon_CSIRO-Mk3L-1-2_abrupt4xCO2_r1i1p1_000101-010012.nc'
fpaths_ta_A4['gi'] = folder+'ta_Amon_GISS-E2-R_abrupt4xCO2_r1i1p1_185001-191912.nc'
fpaths_ta_A4['hg'] = folder+'ta_Amon_HadGEM2-ES_abrupt4xCO2_r1i1p1_185912-191011.nc'
fpaths_ta_A4['ip'] = folder+'ta_Amon_IPSL-CM5A-LR_abrupt4xCO2_r1i1p1_185001-194912.nc'
fpaths_ta_A4['mi'] = folder+'ta_Amon_MIROC-ESM_abrupt4xCO2_r1i1p1_000101-014012.nc'
fpaths_ta_A4['mp'] = folder+'ta_Amon_MPI-ESM-LR_abrput4xCO2_r1i1p1_185001-189912.nc'
fpaths_ta_A4['no'] = folder+'ta_Amon_NorESM1-M_abrupt4xCO2_r1i1p1_000101-010012.nc'



#Specific Humidities
fpaths_hus_PI = OrderedDict()
fpaths_hus_PI['bn'] = folder+'hus_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc'
fpaths_hus_PI['ca'] = folder+'hus_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc'
fpaths_hus_PI['cc'] = folder+'hus_Amon_CCSM4_piControl_r1i1p1_080001-084912.nc'
fpaths_hus_PI['ce'] = folder+'hus_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc'
fpaths_hus_PI['cs'] = folder+'hus_Amon_CSIRO-Mk3L-1-2_piControl_r1i1p1_000101-010012.nc'
fpaths_hus_PI['gi'] = folder+'hus_Amon_GISS-E2-R_piControl_r1i1p1_185001-191912.nc'
fpaths_hus_PI['hg'] = folder+'hus_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc'
fpaths_hus_PI['ip'] = folder+'hus_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-189912.nc'
fpaths_hus_PI['mi'] = folder+'hus_Amon_MIROC-ESM_piControl_r1i1p1_180001-193912.nc'
fpaths_hus_PI['mp'] = folder+'hus_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-188912.nc'
fpaths_hus_PI['no'] = folder+'hus_Amon_NorESM1-M_piControl_r1i1p1_070101-079912.nc'

fpaths_hus_G1 = OrderedDict()
fpaths_hus_G1['bn'] = folder+'hus_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc'
fpaths_hus_G1['ca'] = folder+'hus_Amon_CanESM2_G1_r1i1p1_185001-194912.nc'
fpaths_hus_G1['cc'] = folder+'hus_Amon_CCSM4_G1_r1i1p1_185001-192412.nc'
fpaths_hus_G1['ce'] = folder+'hus_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc'
fpaths_hus_G1['cs'] = folder+'hus_Amon_CSIRO-Mk3L-1-2_G1_r1i1p1_000101-007012.nc'
fpaths_hus_G1['gi'] = folder+'hus_Amon_GISS-E2-R_G1_r1i1p1_185001-191912.nc'
fpaths_hus_G1['hg'] = folder+'hus_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc'
fpaths_hus_G1['ip'] = folder+'hus_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc'
fpaths_hus_G1['mi'] = folder+'hus_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc'
fpaths_hus_G1['mp'] = folder+'hus_Amon_MPI-ESM-LR_G1_r2i1p1_185001-191912.nc'
fpaths_hus_G1['no'] = folder+'hus_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc'

fpaths_hus_A4 = OrderedDict()
fpaths_hus_A4['bn'] = folder+'hus_Amon_BNU-ESM_abrupt4xCO2_r1i1p1_185001-199912.nc'
fpaths_hus_A4['ca'] = folder+'hus_Amon_CanESM2_abrupt4xCO2_r1i1p1_185001-199912.nc'
fpaths_hus_A4['cc'] = folder+'hus_Amon_CCSM4_abrupt4xCO2_r1i1p1_185001-189912.nc'
fpaths_hus_A4['ce'] = folder+'hus_Amon_CESM1-CAM5-1-FV2_abrupt4xCO2_r3i1p1_000101-015012.nc'
fpaths_hus_A4['cs'] = folder+'hus_Amon_CSIRO-Mk3L-1-2_abrupt4xCO2_r1i1p1_000101-010012.nc'
fpaths_hus_A4['gi'] = folder+'hus_Amon_GISS-E2-R_abrupt4xCO2_r1i1p1_185001-191912.nc'
fpaths_hus_A4['hg'] = folder+'hus_Amon_HadGEM2-ES_abrupt4xCO2_r1i1p1_185912-191011.nc'
fpaths_hus_A4['ip'] = folder+'hus_Amon_IPSL-CM5A-LR_abrupt4xCO2_r1i1p1_185001-194912.nc'
fpaths_hus_A4['mi'] = folder+'hus_Amon_MIROC-ESM_abrupt4xCO2_r1i1p1_000101-014012.nc'
fpaths_hus_A4['mp'] = folder+'hus_Amon_MPI-ESM-LR_abrupt4xCO2_r1i1p1_185001-189912.nc'
fpaths_hus_A4['no'] = folder+'hus_Amon_NorESM1-M_abrupt4xCO2_r1i1p1_000101-010012.nc'


#Surface temperatures
fpaths_ts_PI = OrderedDict()
fpaths_ts_PI['bn'] = folder+'ts_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc'
fpaths_ts_PI['ca'] = folder+'ts_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc'
fpaths_ts_PI['cc'] = folder+'ts_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc'
fpaths_ts_PI['ce'] = folder+'ts_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc'
fpaths_ts_PI['cs'] = folder+'ts_Amon_CSIRO-Mk3L-1-2_piControl_r1i1p1_000101-100012.nc'
fpaths_ts_PI['gi'] = folder+'ts_Amon_GISS-E2-R_piControl_r1i1p1_185001-191912.nc'
fpaths_ts_PI['hg'] = folder+'ts_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc'
fpaths_ts_PI['ip'] = folder+'ts_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-279912.nc'
fpaths_ts_PI['mi'] = folder+'ts_Amon_MIROC-ESM_piControl_r1i1p1_180001-233012.nc'
fpaths_ts_PI['mp'] = folder+'ts_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-203512.nc'
fpaths_ts_PI['no'] = folder+'ts_Amon_NorESM1-M_piControl_r1i1p1_070001-115012.nc'

fpaths_ts_G1 = OrderedDict()
fpaths_ts_G1['bn'] = folder+'ts_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc'
fpaths_ts_G1['ca'] = folder+'ts_Amon_CanESM2_G1_r1i1p1_185001-194912.nc'
fpaths_ts_G1['cc'] = folder+'ts_Amon_CCSM4_G1_r1i1p1_185001-192412.nc'
fpaths_ts_G1['ce'] = folder+'ts_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc'
fpaths_ts_G1['cs'] = folder+'ts_Amon_CSIRO-Mk3L-1-2_G1_r1i1p1_000101-007012.nc'
fpaths_ts_G1['gi'] = folder+'ts_Amon_GISS-E2-R_G1_r1i1p1_185001-191912.nc'
fpaths_ts_G1['hg'] = folder+'ts_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc'
fpaths_ts_G1['ip'] = folder+'ts_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc'
fpaths_ts_G1['mi'] = folder+'ts_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc'
fpaths_ts_G1['mp'] = folder+'ts_Amon_MPI-ESM-LR_G1_r2i1p1_185001-191912.nc'
fpaths_ts_G1['no'] = folder+'ts_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc'

fpaths_ts_A4 = OrderedDict()
fpaths_ts_A4['bn'] = folder+'ts_Amon_BNU-ESM_abrupt4xCO2_r1i1p1_185001-199912.nc'
fpaths_ts_A4['ca'] = folder+'ts_Amon_CanESM2_abrupt4xCO2_r1i1p1_185001-199912.nc'
fpaths_ts_A4['cc'] = folder+'ts_Amon_CCSM4_abrupt4xCO2_r1i1p1_185001-200012.nc'
fpaths_ts_A4['ce'] = folder+'ts_Amon_CESM1-CAM5-1-FV2_abrupt4xCO2_r3i1p1_000101-015012.nc'
fpaths_ts_A4['cs'] = folder+'ts_Amon_CSIRO-Mk3L-1-2_abrupt4xCO2_r1i1p1_000101-015012.nc'
fpaths_ts_A4['gi'] = folder+'ts_Amon_GISS-E2-R_abrupt4xCO2_r1i1p1_185001-191912.nc'
fpaths_ts_A4['hg'] = folder+'ts_Amon_HadGEM2-ES_abrupt4xCO2_r1i1p1_185912-191011.nc'
fpaths_ts_A4['ip'] = folder+'ts_Amon_IPSL-CM5A-LR_abrupt4xCO2_r1i1p1_185001-204912.nc'
fpaths_ts_A4['mi'] = folder+'ts_Amon_MIROC-ESM_abrupt4xCO2_r1i1p1_000101-015012.nc'
fpaths_ts_A4['mp'] = folder+'ts_Amon_MPI-ESM-LR_abrupt4xCO2_r1i1p1_185001-199912.nc'
fpaths_ts_A4['no'] = folder+'ts_Amon_NorESM1-M_abrupt4xCO2_r1i1p1_000101-015012.nc'


#Load NetCDF files & create NetCDF Dataset objects
#Temperature
Datasets_ta_G1 = OrderedDict()
for key, value in fpaths_ta_G1.items():
    print('Loading NetCDF file (ta): G1: ' + gmf.modelNames[key]) 
    Datasets_ta_G1[key] = nc4.Dataset(value)
    
Datasets_ta_PI = OrderedDict()    
for key, value in fpaths_ta_PI.items():
    print('Loading NetCDF file (ta): piControl: ' + gmf.modelNames[key]) 
    Datasets_ta_PI[key] = nc4.Dataset(value)    

Datasets_ta_A4 = OrderedDict()    
for key, value in fpaths_ta_A4.items():
    print('Loading NetCDF file (ta): abrupt4xCO2: ' + gmf.modelNames[key]) 
    Datasets_ta_A4[key] = nc4.Dataset(value)   

#Humidity
Datasets_hus_PI = OrderedDict()    
for key, value in fpaths_hus_PI.items():
    print('Loading NetCDF file (hus): piControl: ' + gmf.modelNames[key]) 
    Datasets_hus_PI[key] = nc4.Dataset(value)    

Datasets_hus_G1 = OrderedDict()
for key, value in fpaths_hus_G1.items():
    print('Loading NetCDF file (hus): G1: ' + gmf.modelNames[key]) 
    Datasets_hus_G1[key] = nc4.Dataset(value)
    
Datasets_hus_A4 = OrderedDict()    
for key, value in fpaths_hus_A4.items():
    print('Loading NetCDF file (hus): abrupt4xCO2: ' + gmf.modelNames[key]) 
    Datasets_hus_A4[key] = nc4.Dataset(value)    
    
#Surface temperature
Datasets_ts_G1 = OrderedDict()
for key, value in fpaths_ts_G1.items():
    print('Loading NetCDF file (ts): G1: ' + gmf.modelNames[key]) 
    Datasets_ts_G1[key] = nc4.Dataset(value)
    
Datasets_ts_PI = OrderedDict()    
for key, value in fpaths_ts_PI.items():
    print('Loading NetCDF file (ts): piControl: ' + gmf.modelNames[key]) 
    Datasets_ts_PI[key] = nc4.Dataset(value)    

Datasets_ts_A4 = OrderedDict()    
for key, value in fpaths_ts_A4.items():
    print('Loading NetCDF file (ts): abrupt4xCO2: ' + gmf.modelNames[key]) 
    Datasets_ts_A4[key] = nc4.Dataset(value)  


#Calculate 40-year means in individual months, not taking any spatial means. 
#For G1 and Abrupt4xCO2, the average was over years 11-50 in the Kravitz et al. papers; (10-49 in Python indexing)
#For piControl, it doesn't matter which 40 years, but use 1st 40 years for consistency.


#The standard pressure levels from CMIP5: (hPa)      (Some models need to be subset to this)
levs = np.array([1000, 925, 850, 700, 600, 500, 400, 300, 250, 200, 150, 100, 70, 50, 30, 20, 10]) 
npyFolder = 'npyFiles/' #Locations of the npy files


#Do the averaging and save to npy file

#Temperature
#models_ta = ['bn', 'ca', 'cc', 'ce', 'cs', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']
#models_ta = ['mi']
models_ta = []

#piControl: years 0-39 
monthlyMeans_ta_PI = OrderedDict()
for key in models_ta:
    print('Calculating 40-year monthly mean temperature: piControl: ' + gmf.modelNames[key])
    temp_ta = Datasets_ta_PI[key].variables['ta'][0:40*12,:,:,:]  #Dimensions: time, plev, lat, lon; subset time
    #Subset to the standard CMIP5 pressure levels
    if key == 'mi': #This model reported extra layers in between the standard CESM ones--listed at bottom
        temp_ta = np.ma.concatenate((temp_ta[:, 0:10,:,:], 
                                  temp_ta[:,11:12,:,:],
                                  temp_ta[:,14:15,:,:],
                                  temp_ta[:,17:21,:,:],
                                  temp_ta[:,22:23,:,:]),axis=1)
    else:  #Get rid of any pressure levels beyond 10 hPa
        temp_ta = temp_ta[:,0:17,:,:]
    #Take the averages in each month
    #Angie Pendergrass's kernels also have dimensions of time, plev, lat, lon. So keep these dimensions. 
    #For the returned array, initialize with empty masked array then loop through the 12 months and populate with 40-year averages. 
    s = np.shape(temp_ta)
    numMonths = s[0]
    numLats = s[2]
    numLons = s[3]
    monthlyMeans_ta_PI[key] = np.ma.empty([12, len(levs), numLats, numLons])
    for i in range(0,12):
        monthlyMeans_ta_PI[key][i,:,:,:] = np.mean(temp_ta[i:numMonths:12,:,:,:], axis=0)            
    #Save npy file
    monthlyMeans_ta_PI[key].dump(npyFolder+'monthlyMeansForKernels_ta_PI_'+key+'.npy')
        
        
#G1: years 10-49
monthlyMeans_ta_G1 = OrderedDict()
for key in models_ta:
    print('Calculating 40-year monthly mean temperature: G1: ' + gmf.modelNames[key])
    temp_ta = Datasets_ta_G1[key].variables['ta'][10*12:50*12,:,:,:]  #Dimensions: time, plev, lat, lon; subset time
    #Subset to the standard CMIP5 pressure levels
    if key == 'mi': #This model reported extra layers in between the standard CESM ones--listed at bottom
        temp_ta = np.ma.concatenate((temp_ta[:, 0:10,:,:], 
                                  temp_ta[:,11:12,:,:],
                                  temp_ta[:,14:15,:,:],
                                  temp_ta[:,17:21,:,:],
                                  temp_ta[:,22:23,:,:]),axis=1)
    else:  #Get rid of any pressure levels beyond 10 hPa
        temp_ta = temp_ta[:,0:17,:,:]
    #Take the averages in each month
    #Angie Pendergrass's kernels also have dimensions of time, plev, lat, lon. So keep these dimensions. 
    #For the returned array, initialize with empty masked array then loop through the 12 months and populate with 40-year averages. 
    s = np.shape(temp_ta)
    numMonths = s[0]
    numLats = s[2]
    numLons = s[3]
    monthlyMeans_ta_G1[key] = np.ma.empty([12, len(levs), numLats, numLons])
    for i in range(0,12):
        monthlyMeans_ta_G1[key][i,:,:,:] = np.mean(temp_ta[i:numMonths:12,:,:,:], axis=0)            
    #Save npy file
    monthlyMeans_ta_G1[key].dump(npyFolder+'monthlyMeansForKernels_ta_G1_'+key+'.npy')
        


#aburpt4xCO2: years 10-49
monthlyMeans_ta_A4 = OrderedDict()
for key in models_ta:
    print('Calculating 40-year monthly mean temperature: abrupt4xCO2: ' + gmf.modelNames[key])
    temp_ta = Datasets_ta_A4[key].variables['ta'][10*12:50*12,:,:,:]  #Dimensions: time, plev, lat, lon; subset time
    #Subset to the standard CMIP5 pressure levels
    if key == 'mi': #This model reported extra layers in between the standard CESM ones--listed at bottom
        temp_ta = np.ma.concatenate((temp_ta[:, 0:10,:,:], 
                                  temp_ta[:,11:12,:,:],
                                  temp_ta[:,14:15,:,:],
                                  temp_ta[:,17:21,:,:],
                                  temp_ta[:,22:23,:,:]),axis=1)
    else:  #Get rid of any pressure levels beyond 10 hPa
        temp_ta = temp_ta[:,0:17,:,:]
    #Take the averages in each month
    #Angie Pendergrass's kernels also have dimensions of time, plev, lat, lon. So keep these dimensions. 
    #For the returned array, initialize with empty masked array then loop through the 12 months and populate with 40-year averages. 
    s = np.shape(temp_ta)
    numMonths = s[0]
    numLats = s[2]
    numLons = s[3]
    monthlyMeans_ta_A4[key] = np.ma.empty([12, len(levs), numLats, numLons])
    for i in range(0,12):
        monthlyMeans_ta_A4[key][i,:,:,:] = np.mean(temp_ta[i:numMonths:12,:,:,:], axis=0)            
    #Save npy file
    monthlyMeans_ta_A4[key].dump(npyFolder+'monthlyMeansForKernels_ta_A4_'+key+'.npy')




# Same for humdiity
#models_hus = ['bn', 'ca', 'cc', 'cs', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']
#models_hus = ['mi']
models_hus = ['ce']

#piControl: years 0-39 
monthlyMeans_hus_PI = OrderedDict()
for key in models_hus:
    print('Calculating 40-year monthly mean specific humdidity piControl: ' + gmf.modelNames[key])
    temp_hus = Datasets_hus_PI[key].variables['hus'][0:40*12,:,:,:]  #Dimensions: time, plev, lat, lon; subset time
    #Subset to the standard CMIP5 pressure levels
    if key == 'mi': #This model reported extra layers in between the standard CESM ones--listed at bottom
        temp_hus = np.ma.concatenate((temp_hus[:, 0:10,:,:], 
                                   temp_hus[:,11:12,:,:],
                                   temp_hus[:,14:15,:,:],
                                   temp_hus[:,17:21,:,:],
                                   temp_hus[:,22:23,:,:]),axis=1)
    else:  #Get rid of any pressure levels beyond 10 hPa
        temp_hus = temp_hus[:,0:17,:,:]
    #Take the averages in each month
    #Angie Pendergrass's kernels also have dimensions of time, plev, lat, lon. So keep these dimensions. 
    #For the returned array, initialize with empty masked array then loop through the 12 months and populate with 40-year averages. 
    s = np.shape(temp_hus)
    numMonths = s[0]
    numLats = s[2]
    numLons = s[3]
    monthlyMeans_hus_PI[key] = np.ma.empty([12, len(levs), numLats, numLons])
    for i in range(0,12):
        monthlyMeans_hus_PI[key][i,:,:,:] = np.mean(temp_hus[i:numMonths:12,:,:,:], axis=0)            
    #Save npy file
    monthlyMeans_hus_PI[key].dump(npyFolder+'monthlyMeansForKernels_hus_PI_'+key+'.npy')
        
        
#G1: years 10-49
monthlyMeans_hus_G1 = OrderedDict()
for key in models_hus:
    print('Calculating 40-year monthly mean specific humidity: G1: ' + gmf.modelNames[key])
    temp_hus = Datasets_hus_G1[key].variables['hus'][10*12:50*12,:,:,:]  #Dimensions: time, plev, lat, lon; subset time
    #Subset to the standard CMIP5 pressure levels
    if key == 'mi': #This model reported extra layers in between the standard CESM ones--listed at bottom
        temp_hus = np.ma.concatenate((temp_hus[:, 0:10,:,:], 
                                   temp_hus[:,11:12,:,:],
                                   temp_hus[:,14:15,:,:],
                                   temp_hus[:,17:21,:,:],
                                   temp_hus[:,22:23,:,:]),axis=1)
    else:  #Get rid of any pressure levels beyond 10 hPa
        temp_hus = temp_hus[:,0:17,:,:]
    #Take the averages in each month
    #Angie Pendergrass's kernels also have dimensions of time, plev, lat, lon. So keep these dimensions. 
    #For the returned array, initialize with empty masked array then loop through the 12 months and populate with 40-year averages. 
    s = np.shape(temp_hus)
    numMonths = s[0]
    numLats = s[2]
    numLons = s[3]
    monthlyMeans_hus_G1[key] = np.ma.empty([12, len(levs), numLats, numLons])
    for i in range(0,12):
        monthlyMeans_hus_G1[key][i,:,:,:] = np.mean(temp_hus[i:numMonths:12,:,:,:], axis=0)            
    #Save npy file (all masked arrays)
    monthlyMeans_hus_G1[key].dump(npyFolder+'monthlyMeansForKernels_hus_G1_'+key+'.npy')
        


#aburpt4xCO2: years 10-49
monthlyMeans_hus_A4 = OrderedDict()
for key in models_hus:
    print('Calculating 40-year monthly mean specific humidity: abrupt4xCO2: ' + gmf.modelNames[key])
    temp_hus = Datasets_hus_A4[key].variables['hus'][10*12:50*12,:,:,:]  #Dimensions: time, plev, lat, lon; subset time
    #Subset to the standard CMIP5 pressure levels
    if key == 'mi': #This model reported extra layers in between the standard CESM ones--listed at bottom
        temp_hus = np.ma.concatenate((temp_hus[:, 0:10,:,:], 
                                   temp_hus[:,11:12,:,:],
                                   temp_hus[:,14:15,:,:],
                                   temp_hus[:,17:21,:,:],
                                   temp_hus[:,22:23,:,:]),axis=1)
    else:  #Get rid of any pressure levels beyond 10 hPa
        temp_hus = temp_hus[:,0:17,:,:]
    #Take the averages in each month
    #Angie Pendergrass's kernels also have dimensions of time, plev, lat, lon. So keep these dimensions. 
    #For the returned array, initialize with empty masked array then loop through the 12 months and populate with 40-year averages. 
    s = np.shape(temp_hus)
    numMonths = s[0]
    numLats = s[2]
    numLons = s[3]
    monthlyMeans_hus_A4[key] = np.ma.empty([12, len(levs), numLats, numLons])
    for i in range(0,12):
        monthlyMeans_hus_A4[key][i,:,:,:] = np.mean(temp_hus[i:numMonths:12,:,:,:], axis=0)            
    #Save npy file
    monthlyMeans_hus_A4[key].dump(npyFolder+'monthlyMeansForKernels_hus_A4_'+key+'.npy')




#40-year means for surface temperature--dimensions time, lat, lon: simpler. 
#Need to interpolate the HadGEM2-ES temperatures to the same grid as ta (at midpoints) so it will match the kernel.


#models_ts = ['bn', 'ca', 'cc', 'ce', 'cs', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']
models_ts = []


#piControl: years 0-39 
monthlyMeans_ts_PI = OrderedDict()
for key in models_ts:
    print('Calculating 40-year monthly mean surface temerature in piControl: ' + gmf.modelNames[key])
    temp_ts = Datasets_ts_PI[key].variables['ts'][0:40*12,:,:]  #Dimensions: time, lat, lon; subset time
    print(np.shape(temp_ts))
    #Interpolate the HadGEM temperatures to midpoint latitudes 
    #(ta grid is halfway between ts grid points, so can just take averages of adjacent points)
    if key == 'hg':
        temp_ts = (temp_ts[:,1:,:]+temp_ts[:,:-1,:])/2.
        print(np.shape(temp_ts))
    #Take the averages in each month
    #For the returned array, initialize with empty masked array then loop through the 12 months and populate with 40-year averages. 
    s = np.shape(temp_ts)
    numMonths = s[0]
    print(numMonths)
    numLats = s[1]
    print(numLats)
    numLons = s[2]
    print(numLons)
    monthlyMeans_ts_PI[key] = np.ma.empty([12, numLats, numLons])
    for i in range(0,12):
        monthlyMeans_ts_PI[key][i,:,:] = np.mean(temp_ts[i:numMonths:12,:,:], axis=0)            
    #Save npy file
    monthlyMeans_ts_PI[key].dump(npyFolder+'monthlyMeansForKernels_ts_PI_'+key+'.npy')


#G1: years 10-49
monthlyMeans_ts_G1 = OrderedDict()
for key in models_ts:
    print('Calculating 40-year monthly mean surface temerature in G1: ' + gmf.modelNames[key])
    temp_ts = Datasets_ts_G1[key].variables['ts'][10*12:50*12,:,:]  #Dimensions: time, lat, lon; subset time
    #Interpolate the HadGEM temperatures to midpoint latitudes 
    #(ta grid is halfway between ts grid points, so can just take averages of adjacent points)
    if key == 'hg':
        temp_ts = (temp_ts[:,1:,:]+temp_ts[:,:-1,:])/2.
    #Take the averages in each month
    #For the returned array, initialize with empty masked array then loop through the 12 months and populate with 40-year averages. 
    s = np.shape(temp_ts)
    numMonths = s[0]
    numLats = s[1]
    numLons = s[2]
    monthlyMeans_ts_G1[key] = np.ma.empty([12, numLats, numLons])
    for i in range(0,12):
        monthlyMeans_ts_G1[key][i,:,:] = np.mean(temp_ts[i:numMonths:12,:,:], axis=0)            
    #Save npy file
    monthlyMeans_ts_G1[key].dump(npyFolder+'monthlyMeansForKernels_ts_G1_'+key+'.npy')


#aburpt4xCO2: years 10-49
monthlyMeans_ts_A4 = OrderedDict()
for key in models_ts:
    print('Calculating 40-year monthly mean surface temerature in abrupt4xCO2: ' + gmf.modelNames[key])
    temp_ts = Datasets_ts_A4[key].variables['ts'][10*12:50*12,:,:]  #Dimensions: time, lat, lon; subset time
    #Interpolate the HadGEM temperatures to midpoint latitudes 
    #(ta grid is halfway between ts grid points, so can just take averages of adjacent points)
    if key == 'hg':
        temp_ts = (temp_ts[:,1:,:]+temp_ts[:,:-1,:])/2.
        print(np.shape(temp_ts))
        print(np.shape(temp_ts[:,1:,:]))
        print(np.shape(temp_ts[:,:-1,:]))
        print(np.shape((temp_ts[:,1:,:]+temp_ts[:,:-1,:])/2.))
    #Take the averages in each month
    #For the returned array, initialize with empty masked array then loop through the 12 months and populate with 40-year averages. 
    s = np.shape(temp_ts)
    numMonths = s[0]
    numLats = s[1]
    numLons = s[2]
    monthlyMeans_ts_A4[key] = np.ma.empty([12, numLats, numLons])
    for i in range(0,12):
        monthlyMeans_ts_A4[key][i,:,:] = np.mean(temp_ts[i:numMonths:12,:,:], axis=0)            
    #Save npy file
    monthlyMeans_ts_A4[key].dump(npyFolder+'monthlyMeansForKernels_ts_A4_'+key+'.npy')




# Take differences between runs and save these in npy files as well
monthlyMeans_ta_G1minusPI = OrderedDict()
monthlyMeans_ta_A4minusPI = OrderedDict()

for key in models_ta:
    monthlyMeans_ta_G1minusPI[key] = monthlyMeans_ta_G1[key] - monthlyMeans_ta_PI[key]
    monthlyMeans_ta_A4minusPI[key] = monthlyMeans_ta_A4[key] - monthlyMeans_ta_PI[key]
    monthlyMeans_ta_G1minusPI[key].dump(npyFolder+'monthlyMeansForKernels_ta_G1minusPI_'+key+'.npy')    
    monthlyMeans_ta_A4minusPI[key].dump(npyFolder+'monthlyMeansForKernels_ta_A4minusPI_'+key+'.npy')    
        

monthlyMeans_hus_G1minusPI = OrderedDict()
monthlyMeans_hus_A4minusPI = OrderedDict()

for key in models_hus:
    monthlyMeans_hus_G1minusPI[key] = monthlyMeans_hus_G1[key] - monthlyMeans_hus_PI[key]
    monthlyMeans_hus_A4minusPI[key] = monthlyMeans_hus_A4[key] - monthlyMeans_hus_PI[key]
    monthlyMeans_hus_G1minusPI[key].dump(npyFolder+'monthlyMeansForKernels_hus_G1minusPI_'+key+'.npy')    
    monthlyMeans_hus_A4minusPI[key].dump(npyFolder+'monthlyMeansForKernels_hus_A4minusPI_'+key+'.npy')    
    

monthlyMeans_ts_G1minusPI = OrderedDict()
monthlyMeans_ts_A4minusPI = OrderedDict()

for key in models_ts:
    monthlyMeans_ts_G1minusPI[key] = monthlyMeans_ts_G1[key] - monthlyMeans_ts_PI[key]
    monthlyMeans_ts_A4minusPI[key] = monthlyMeans_ts_A4[key] - monthlyMeans_ts_PI[key]
    monthlyMeans_ts_G1minusPI[key].dump(npyFolder+'monthlyMeansForKernels_ts_G1minusPI_'+key+'.npy')    
    monthlyMeans_ts_A4minusPI[key].dump(npyFolder+'monthlyMeansForKernels_ts_A4minusPI_'+key+'.npy')    
        
        
        
        
    