# Script to plot zonal means of clouds in G1, piControl and abrupt4xCO2 in GeoMIP models.
# This will include raw means as well as differences for individual models. 

import numpy as np
import matplotlib.pyplot as plt
import netCDF4 as nc4
import geomipFunctions as gmf
import os
from collections import OrderedDict


#Paths for the files
#HadGEM and CSIRO calculations done in separate scripts.
folder = '../nobackup/' 
#Cloud fractions
fpaths_cl_G1 = OrderedDict()
#fpaths_cl_G1['bn'] = folder+'cl_Amon_BNU-ESM_G1_r1i1p1_185001-190012.nc'
fpaths_cl_G1['bn'] = folder+'cl_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc'
fpaths_cl_G1['ca'] = folder+'cl_Amon_CanESM2_G1_r1i1p1_185001-194912.nc'
fpaths_cl_G1['cc'] = folder+'cl_Amon_CCSM4_G1_r1i1p1_185001-189912.nc'
fpaths_cl_G1['ce'] = folder+'cl_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc'
fpaths_cl_G1['gi'] = folder+'cl_Amon_GISS-E2-R_G1_r1i1p1_185001-191912.nc'
fpaths_cl_G1['ip'] = folder+'cl_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc'
fpaths_cl_G1['mi'] = folder+'cl_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc'
fpaths_cl_G1['mp'] = folder+'cl_Amon_MPI-ESM-LR_G1_r2i1p1_185001-191912.nc' 
fpaths_cl_G1['no'] = folder+'cl_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc'

fpaths_cl_PI = OrderedDict()
fpaths_cl_PI['bn'] = folder+'cl_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc'
fpaths_cl_PI['ca'] = folder+'cl_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc'
fpaths_cl_PI['cc'] = folder+'cl_Amon_CCSM4_piControl_r1i1p1_080001-084912.nc'
fpaths_cl_PI['ce'] = folder+'cl_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc'
fpaths_cl_PI['gi'] = folder+'cl_Amon_GISS-E2-R_piControl_r1i1p1_185001-191912.nc'
fpaths_cl_PI['ip'] = folder+'cl_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-189912.nc'
fpaths_cl_PI['mi'] = folder+'cl_Amon_MIROC-ESM_piControl_r1i1p1_180001-185912.nc'
fpaths_cl_PI['mp'] = folder+'cl_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-189912.nc'
fpaths_cl_PI['no'] = folder+'cl_Amon_NorESM1-M_piControl_r1i1p1_070101-079912.nc'

fpaths_cl_A4 = OrderedDict()
fpaths_cl_A4['bn'] = folder+'cl_Amon_BNU-ESM_abrupt4xCO2_r1i1p1_185001-199912.nc'
fpaths_cl_A4['ca'] = folder+'cl_Amon_CanESM2_abrupt4xCO2_r1i1p1_185001-199912.nc'
fpaths_cl_A4['cc'] = folder+'cl_Amon_CCSM4_abrupt4xCO2_r1i1p1_185001-189912.nc'
fpaths_cl_A4['ce'] = folder+'cl_Amon_CESM1-CAM5-1-FV2_abrupt4xCO2_r3i1p1_000101-015012.nc'
fpaths_cl_A4['gi'] = folder+'cl_Amon_GISS-E2-R_abrupt4xCO2_r1i1p1_185001-191912.nc'
fpaths_cl_A4['ip'] = folder+'cl_Amon_IPSL-CM5A-LR_abrupt4xCO2_r1i1p1_185001-194912.nc'
fpaths_cl_A4['mi'] = folder+'cl_Amon_MIROC-ESM_abrupt4xCO2_r1i1p1_000101-006012.nc'
fpaths_cl_A4['mp'] = folder+'cl_Amon_MPI-ESM-LR_abrupt4xCO2_r1i1p1_185001-189912.nc'
fpaths_cl_A4['no'] = folder+'cl_Amon_NorESM1-M_abrupt4xCO2_r1i1p1_000101-010012.nc'

#Load model lats and lons
#Not sure yet which version (ta or tas grid) I will need for HadGEM2-ES model. 
#It's the tas grid.
modelLats = np.load('npyFiles/modelLats.npy').item()
modelLons = np.load('npyFiles/modelLons.npy').item()  




#Interpolation takes a long time, so I want to be able to load from npy files, and to control whether this loading
#takes place by individual model (so that more models can be added later without redoing the calculations for the 
#earlier models). Therefore need a dictionary of "loadFromFile" booleans. I don't think I'll split it up by experiment though. 
#Wait--I need to do that to match the dictionaries of file names, etc.
loadNpyFile_cl_G1 = OrderedDict()
loadNpyFile_cl_G1['bn'] = False
loadNpyFile_cl_G1['ca'] = False
loadNpyFile_cl_G1['cc'] = False
loadNpyFile_cl_G1['ce'] = False
loadNpyFile_cl_G1['cs'] = False
loadNpyFile_cl_G1['gi'] = False
loadNpyFile_cl_G1['hg'] = False
loadNpyFile_cl_G1['ip'] = False
loadNpyFile_cl_G1['mi'] = False
loadNpyFile_cl_G1['mp'] = False
loadNpyFile_cl_G1['no'] = False


loadNpyFile_cl_PI = OrderedDict()
loadNpyFile_cl_PI['bn'] = False
loadNpyFile_cl_PI['ca'] = False
loadNpyFile_cl_PI['cc'] = False
loadNpyFile_cl_PI['ce'] = False
loadNpyFile_cl_PI['cs'] = False
loadNpyFile_cl_PI['gi'] = False
loadNpyFile_cl_PI['hg'] = False
loadNpyFile_cl_PI['ip'] = False
loadNpyFile_cl_PI['mi'] = False
loadNpyFile_cl_PI['mp'] = False
loadNpyFile_cl_PI['no'] = False

loadNpyFile_cl_A4 = OrderedDict()
loadNpyFile_cl_A4['bn'] = False
loadNpyFile_cl_A4['ca'] = False
loadNpyFile_cl_A4['cc'] = False
loadNpyFile_cl_A4['ce'] = False
loadNpyFile_cl_A4['cs'] = False
loadNpyFile_cl_A4['gi'] = False
loadNpyFile_cl_A4['hg'] = False
loadNpyFile_cl_A4['ip'] = False
loadNpyFile_cl_A4['mi'] = False
loadNpyFile_cl_A4['mp'] = False
loadNpyFile_cl_A4['no'] = False


#Load NetCDF files & create NetCDF Dataset objects
Datasets_cl_G1 = OrderedDict()
for key, value in fpaths_cl_G1.items():
    #if not(loadNpyFile_cl_G1[key]):       #Still need at these objects for latitudes for plotting
    print('Loading NetCDF file  (cl): G1: ' + gmf.modelNames[key]) 
    Datasets_cl_G1[key] = nc4.Dataset(value)
    
Datasets_cl_PI = OrderedDict()    
for key, value in fpaths_cl_PI.items():
    print('Loading NetCDF file (cl): piControl: ' + gmf.modelNames[key]) 
    Datasets_cl_PI[key] = nc4.Dataset(value)    

Datasets_cl_A4 = OrderedDict()    
for key, value in fpaths_cl_A4.items():
    print('Loading NetCDF file (cl): abrupt4xCO2: ' + gmf.modelNames[key]) 
    Datasets_cl_A4[key] = nc4.Dataset(value)    
    


#Calculate zonal means, averaged over 40 years, interpolated to the CMIP5 pressure grid. 
#For G1 and Abrupt4xCO2, the average was over years 11-50 in the Kravitz et al. papers; (10-49 in Python indexing)
#For piControl, it doesn't matter which 40 years. 
    
#The standard pressure levels from CMIP5: (hPa)    
levs = np.array([10, 20, 30, 50, 70, 100, 150, 200, 250, 300, 400, 500, 600, 700, 850, 925, 1000]) 
npyFolder = 'npyFiles/' #Locations of the npy files

#Do the pressure interpolation and zonal mean calculation and save to npy file, or load from existing npy files
modelsForCalculatingOrLoading = ['bn', 'ca', 'cc', 'ce', 'cs', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']

#G1: years 10-49 (if year indices start from 0)
zonalMeans_cl_G1 = OrderedDict()
for key in modelsForCalculatingOrLoading:
    if not(loadNpyFile_cl_G1[key]):
        print('Calculating zonal mean cloud fraction: G1: ' + gmf.modelNames[key])
        if key == 'gi': #GISS model: Already in pressure coordinates
            zonalMeans_cl_G1[key] = gmf.zonalMeanMultiYearPressureCoordinates(Datasets_cl_G1[key], 'cl', 10, 49)
            zonalMeans_cl_G1[key] = zonalMeans_cl_G1[key][0:17,:] #Get rid of any pressure levels beyond 10 hPa
            zonalMeans_cl_G1[key] = np.flipud(zonalMeans_cl_G1[key]) #Reverse order of pressure levels to match other models
        else:
            zonalMeans_cl_G1[key] = gmf.zonalMeanMultiYearPressureVaryingVariable(Datasets_cl_G1[key], 'cl', levs, 10, 49, key)
        np.save(npyFolder+'zonalMeans_cl_G1_'+key+'.npy', zonalMeans_cl_G1[key])
    else:
        print('Loading zonal mean cloud fraction from npy file: G1: ' + gmf.modelNames[key])
        zonalMeans_cl_G1[key] = np.load(npyFolder+'zonalMeans_cl_G1_'+key+'.npy')
    if key == 'ce': #cl was reported from 0 to 1 instead of %
        zonalMeans_cl_G1[key] = zonalMeans_cl_G1[key]*100.
        
#piControl: years 0-39 (doesn't matter which 40 years in this case)        
zonalMeans_cl_PI = OrderedDict()
for key in modelsForCalculatingOrLoading:
    if not(loadNpyFile_cl_PI[key]):
        print('Calculating zonal mean cloud fraction: piControl: ' + gmf.modelNames[key])
        if key == 'gi': #GISS model: Already in pressure coordinates
            zonalMeans_cl_PI[key] = gmf.zonalMeanMultiYearPressureCoordinates(Datasets_cl_PI[key], 'cl', 0, 39)
            zonalMeans_cl_PI[key] = zonalMeans_cl_PI[key][0:17,:] #Get rid of any pressure levels beyond 10 hPa
            zonalMeans_cl_PI[key] = np.flipud(zonalMeans_cl_PI[key]) #Reverse order of pressure levels to match other models
        else:
            zonalMeans_cl_PI[key] = gmf.zonalMeanMultiYearPressureVaryingVariable(Datasets_cl_PI[key], 'cl', levs, 0, 39, key)
        np.save(npyFolder+'zonalMeans_cl_PI_'+key+'.npy', zonalMeans_cl_PI[key])
    else:
        print('Loading zonal mean cloud fraction from npy file: piControl: ' + gmf.modelNames[key])
        zonalMeans_cl_PI[key] = np.load(npyFolder+'zonalMeans_cl_PI_'+key+'.npy')    
    if key == 'ce': #cl was reported from 0 to 1 instead of %
        zonalMeans_cl_PI[key] = zonalMeans_cl_PI[key]*100.
        
#abrupt4xCO2: years 10-49        
zonalMeans_cl_A4 = OrderedDict()
for key in modelsForCalculatingOrLoading:
    if not(loadNpyFile_cl_A4[key]):
        print('Calculating zonal mean cloud fraction: abrupt4xCO2: ' + gmf.modelNames[key])
        if key == 'gi': #GISS model: Already in pressure coordinates
            zonalMeans_cl_A4[key] = gmf.zonalMeanMultiYearPressureCoordinates(Datasets_cl_A4[key], 'cl', 10, 49)
            zonalMeans_cl_A4[key] = zonalMeans_cl_A4[key][0:17,:] #Get rid of any pressure levels beyond 10 hPa
            zonalMeans_cl_A4[key] = np.flipud(zonalMeans_cl_A4[key]) #Reverse order of pressure levels to match other models
        else:
            zonalMeans_cl_A4[key] = gmf.zonalMeanMultiYearPressureVaryingVariable(Datasets_cl_A4[key], 'cl', levs, 10, 49, key)
        np.save(npyFolder+'zonalMeans_cl_A4_'+key+'.npy', zonalMeans_cl_A4[key])
    else:
        print('Loading zonal mean cloud fraction from npy file: abrupt4xCO2: ' + gmf.modelNames[key])
        zonalMeans_cl_A4[key] = np.load(npyFolder+'zonalMeans_cl_A4_'+key+'.npy')   
    if key == 'ce': #cl was reported from 0 to 1 instead of %
        zonalMeans_cl_A4[key] = zonalMeans_cl_A4[key]*100.        

print('Finished zonal mean/vertical interpolation calculations or loading.')





    

#11-panel figure, with colorbar in the 12th panel

#For HadGEM2-ES, will have to plot in height coordinates instead of pressure. So should put y-axis labels on every panel, 
#unlike the versions of this for temperature and humidity.

keys = ['bn', 'ca', 'cc', 'ce', 'cs', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']
(w,h) = plt.figaspect(0.7)*1.9
fig = plt.figure(500, figsize=(w,h))
for i in range(11): 
    key = keys[i]
    print('Suplot # ' + str(i+1))
    plt.subplot(3,4,i+1)
    lat = modelLats[key]
    plotData = zonalMeans_cl_G1[key] - zonalMeans_cl_PI[key]
    if(key == 'bn' or key == 'cc'):
        plotData = plotData*100 #These models gave cloud fraction as fraction instead of %, not corrected earlier
    if not(i == 6):
        thePlot = plt.contourf(np.sin(lat*np.pi/180.), levs, plotData, levels=np.linspace(-3.,3.,12), cmap=plt.cm.RdBu_r, extend='both')        
        plt.gca().invert_yaxis()
    else: #HadGEM model--height coords. 
        zLevels = np.flipud(np.array([25, 100, 300, 525, 750, 1000, 1500, 2250, 3000, 4000, 5000, 6000, 7000, 
                              8500, 10000, 14000, 18000, 22000, 26000, 30000, 34000, 38000]))
        print(np.shape(lat))
        print(np.shape(zLevels))
        print(np.shape(plotData))
        thePlot = plt.contourf(np.sin(lat*np.pi/180.), np.exp(-1./8000.*zLevels), plotData, 
                               levels=np.linspace(-3.,3.,12), cmap=plt.cm.RdBu_r, extend='both')
    if i in range(8,11):
        plt.xlabel('Latitude ($^\circ$N)', fontsize=15)
    plt.xticks(np.sin(np.array([-90, -30, 0, 30, 90])*np.pi/180.),  ['-90', '-30', '0', '30', '90'])
    if not(i == 6): #Y axis labels for pressure coordinates
        plt.ylabel('Pressure (hPa)', fontsize=15) 
        plt.yticks([1000, 800, 600, 400, 200, 0])
    else: #Y axis labels for height coordinates
        plt.ylabel('Height (m)', fontsize=15)
        plt.yticks(np.exp(-1./8000.*np.array([0., 1500., 3250., 6500., 10000, 20000])), 
                   ['0', '1500', '3250', '6500', '10000', '20000'])
        plt.axis([-1., 1., np.exp(-1./8000.*0), np.exp(-1./8000.*20000)])
    plt.title(gmf.modelNames[keys[i]], fontsize=15)
    plt.gca().set_axis_bgcolor('k') 
#Colorbar
cbar = plt.colorbar(thePlot, cax=fig.add_axes([0.78, 0.18, 0.2, 0.03]), ticks=np.linspace(-3., 3., 7), orientation='horizontal')
cbar.ax.set_xlabel('Cloud fraction change (%)', fontsize=15)
cbar.ax.tick_params(labelsize=15)
plt.tight_layout()
plt.savefig('plots/cl_zonalMean_diff_ACP_paperVersion_flipped.pdf')


plt.show()    

