# Script to calculate and plot the correction factor applied to rlut, so that it can also be applied to rlutcs (after saving to npy files). 

# This is for the NCAR family of models (CCSM4, CESM, NorESM-1) that had the rlut correction issue. 

# Paths to files, before (old) and after (new) correction applied

import numpy as np
import netCDF4 as nc4
from collections import OrderedDict

fpaths_rlut_old_PI = {'bn': '../nobackup/BNU_rlut_uncorrected/rlut_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc',
                      'cc': '../nobackup/unusedFragments/incorrectCCSM4_rlut/rlut_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc',
                      'ce': '../nobackup/CESM_rlut_uncorrected/rlut_Amon_CESM-CAM5.1-FV_piControl_r1i1p1_000101-005012.nc',
                      'no': '../nobackup/NorESM_rlut_uncorrected/rlut_Amon_NorESM1-M_piControl_r1i1p1_070101-115012.nc'}

fpaths_rlut_old_G1 = {'bn': '../nobackup/BNU_rlut_uncorrected/rlut_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc',
                      'cc': '../nobackup/unusedFragments/incorrectCCSM4_rlut/rlut_Amon_CCSM4_G1_r1i1p1_185001-192412.nc', 
                      'ce': '../nobackup/CESM_rlut_uncorrected/rlut_Amon_CESM-CAM5.1-FV_G1_r1i1p1_000101-005912.nc',
                      'no': '../nobackup/NorESM_rlut_uncorrected/rlut_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc'}
                      
fpaths_rlut_old_A4 = {'bn': '../nobackup/BNU_rlut_uncorrected/rlut_Amon_BNU-ESM_abrupt4xCO2_r1i1p1_185001-199912.nc',
                      'cc': '../nobackup/unusedFragments/incorrectCCSM4_rlut/rlut_Amon_CCSM4_abrupt4xCO2_r1i1p1_185001-200012.nc',
                      'ce': '../nobackup/CESM_rlut_uncorrected/rlut_Amon_CESM1-CAM5-1-FV2_abrupt4xCO2_r3i1p1_000101-015012.nc',
                      'no': '../nobackup/NorESM_rlut_uncorrected/rlut_Amon_NorESM1-M_abrupt4xCO2_r1i1p1_000101-015012.nc'}

fpaths_rlut_new_PI = {'bn': '../nobackup/rlut_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc',
                      'cc': '../nobackup/CCSM4_fluxes_from_NCAR/rlut_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc',
                      'ce': '../nobackup/CESM_corrected/rlut_Amon_CESM1-CAM5.1-FV2_piControl_r1i1p1_000101-005012.nc',
                      'no': '../nobackup/NorESM_corrected_rlut/rlut_Amon_NorESM1-M_piControl_r1i1p1_070101-120012.nc'}
                      
fpaths_rlut_new_G1 = {'bn': '../nobackup/rlut_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc',
                      'cc': '../nobackup/CCSM4_fluxes_from_NCAR/rlut_Amon_CCSM4_G1_r1i1p1_185001-192412.nc',
                      'ce': '../nobackup/CESM_corrected/rlut_Amon_CESM1-CAM5.1-FV2_G1_r1i1p1_000101-006012.nc',
                      'no': '../nobackup/NorESM_corrected_rlut/rlut_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc'}

fpaths_rlut_new_A4 = {'bn': '../nobackup/rlut_Amon_BNU-ESM_abrupt4xCO2_r1i1p1_185001-199912.nc',
                      'cc': '../nobackup/CCSM4_fluxes_from_NCAR/rlut_Amon_CCSM4_abrupt4xCO2_r1i1p1_185001-200012.nc',
                      'ce': '../nobackup/CESM_corrected/rlut_Amon_CESM1-CAM5.1-FV2_abrupt4xCO2_r3i1p1_000101-015012.nc',
                      'no': '../nobackup/NorESM_corrected_rlut/rlut_Amon_NorESM1-M_abrupt4xCO2_r1i1p1_000101-015012.nc'}
                      
#Create netCDF Dataset objects
Datasets_rlut_old_PI = OrderedDict()                      
for key in fpaths_rlut_old_PI.keys():
    Datasets_rlut_old_PI[key] = nc4.Dataset(fpaths_rlut_old_PI[key])
    
Datasets_rlut_old_G1 = OrderedDict()                      
for key in fpaths_rlut_old_G1.keys():
    Datasets_rlut_old_G1[key] = nc4.Dataset(fpaths_rlut_old_G1[key])
    
Datasets_rlut_old_A4 = OrderedDict()                      
for key in fpaths_rlut_old_A4.keys():
    Datasets_rlut_old_A4[key] = nc4.Dataset(fpaths_rlut_old_A4[key])

#new    
Datasets_rlut_new_PI = OrderedDict()                      
for key in fpaths_rlut_new_PI.keys():
    print(key)
    Datasets_rlut_new_PI[key] = nc4.Dataset(fpaths_rlut_new_PI[key])
    
Datasets_rlut_new_G1 = OrderedDict()                      
for key in fpaths_rlut_new_G1.keys():
    Datasets_rlut_new_G1[key] = nc4.Dataset(fpaths_rlut_new_G1[key])
    
Datasets_rlut_new_A4 = OrderedDict()                      
for key in fpaths_rlut_new_A4.keys():
    Datasets_rlut_new_A4[key] = nc4.Dataset(fpaths_rlut_new_A4[key])    
    
keys = ['bn', 'cc', 'ce', 'no']
    
#Take differences
correction_PI = OrderedDict()
for key in keys:
    correction_PI[key] = Datasets_rlut_new_PI[key].variables['rlut'][0:40*12,:,:] - Datasets_rlut_old_PI[key].variables['rlut'][0:40*12,:,:]
    
correction_G1 = OrderedDict()
for key in keys:
    correction_G1[key] = Datasets_rlut_new_G1[key].variables['rlut'][10*12:50*12,:,:] - Datasets_rlut_old_G1[key].variables['rlut'][10*12:50*12,:,:]

correction_A4 = OrderedDict()
for key in keys:
    correction_A4[key] = Datasets_rlut_new_A4[key].variables['rlut'][10*12:50*12,:,:] - Datasets_rlut_old_A4[key].variables['rlut'][10*12:50*12,:,:]    
    
#Imported the script in ipython and made contour plots. Correction is between about -1 and 5 W/m^2, depending on latitude, season, model and run. 
    
#Save results as npy files, so they can be loaded and used in other scripts. 
np.save('npyFiles/rlut_correction_PI.npy', correction_PI)
np.save('npyFiles/rlut_correction_G1.npy', correction_G1)
np.save('npyFiles/rlut_correction_A4.npy', correction_A4)

    
