# Script to plot changes in humidity (hus) on zonal mean-height basis. 
#
# This is intended to see how this might help explain changes in clouds (particularly high clouds) 
# that occur in GeoMIP. 
#
# I have these files for all GeoMIP G1 models except CESM, EC-Earth, and HadCM3. 
#
# Rick Russotto
# Started 2017-09-25
#
# 3-28-18: adding CESM model

import numpy as np
import matplotlib.pyplot as plt
import netCDF4 as nc4
import geomipFunctions as gmf
import os
from collections import OrderedDict



#Paths for the files
folder = '../nobackup/' 

fpaths_hus_PI = OrderedDict()
fpaths_hus_PI['bn'] = folder+'hus_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc'
fpaths_hus_PI['ca'] = folder+'hus_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc'
fpaths_hus_PI['cc'] = folder+'hus_Amon_CCSM4_piControl_r1i1p1_080001-084912.nc'
fpaths_hus_PI['ce'] = folder+'hus_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc'
fpaths_hus_PI['cs'] = folder+'hus_Amon_CSIRO-Mk3L-1-2_piControl_r1i1p1_000101-010012.nc'
fpaths_hus_PI['gi'] = folder+'hus_Amon_GISS-E2-R_piControl_r1i1p1_185001-191912.nc'
fpaths_hus_PI['hg'] = folder+'hus_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc'
fpaths_hus_PI['ip'] = folder+'hus_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-189912.nc'
fpaths_hus_PI['mi'] = folder+'hus_Amon_MIROC-ESM_piControl_r1i1p1_180001-193912.nc'
fpaths_hus_PI['mp'] = folder+'hus_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-188912.nc'
fpaths_hus_PI['no'] = folder+'hus_Amon_NorESM1-M_piControl_r1i1p1_070101-079912.nc'

fpaths_hus_G1 = OrderedDict()
#fpaths_hus_G1['bn'] = folder+'hus_Amon_BNU-ESM_G1_r1i1p1_185001-190012.nc'
fpaths_hus_G1['bn'] = folder+'hus_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc'
fpaths_hus_G1['ca'] = folder+'hus_Amon_CanESM2_G1_r1i1p1_185001-194912.nc'
fpaths_hus_G1['cc'] = folder+'hus_Amon_CCSM4_G1_r1i1p1_185001-192412.nc'
fpaths_hus_G1['ce'] = folder+'hus_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc'
fpaths_hus_G1['cs'] = folder+'hus_Amon_CSIRO-Mk3L-1-2_G1_r1i1p1_000101-007012.nc'
fpaths_hus_G1['gi'] = folder+'hus_Amon_GISS-E2-R_G1_r1i1p1_185001-191912.nc'
fpaths_hus_G1['hg'] = folder+'hus_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc'
fpaths_hus_G1['ip'] = folder+'hus_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc'
fpaths_hus_G1['mi'] = folder+'hus_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc'
fpaths_hus_G1['mp'] = folder+'hus_Amon_MPI-ESM-LR_G1_r2i1p1_185001-191912.nc'
fpaths_hus_G1['no'] = folder+'hus_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc'

fpaths_hus_A4 = OrderedDict()
fpaths_hus_A4['bn'] = folder+'hus_Amon_BNU-ESM_abrupt4xCO2_r1i1p1_185001-199912.nc'
fpaths_hus_A4['ca'] = folder+'hus_Amon_CanESM2_abrupt4xCO2_r1i1p1_185001-199912.nc'
fpaths_hus_A4['cc'] = folder+'hus_Amon_CCSM4_abrupt4xCO2_r1i1p1_185001-189912.nc'
fpaths_hus_A4['ce'] = folder+'hus_Amon_CESM1-CAM5-1-FV2_abrupt4xCO2_r3i1p1_000101-015012.nc'
fpaths_hus_A4['cs'] = folder+'hus_Amon_CSIRO-Mk3L-1-2_abrupt4xCO2_r1i1p1_000101-010012.nc'
fpaths_hus_A4['gi'] = folder+'hus_Amon_GISS-E2-R_abrupt4xCO2_r1i1p1_185001-191912.nc'
fpaths_hus_A4['hg'] = folder+'hus_Amon_HadGEM2-ES_abrupt4xCO2_r1i1p1_185912-191011.nc'
fpaths_hus_A4['ip'] = folder+'hus_Amon_IPSL-CM5A-LR_abrupt4xCO2_r1i1p1_185001-194912.nc'
fpaths_hus_A4['mi'] = folder+'hus_Amon_MIROC-ESM_abrupt4xCO2_r1i1p1_000101-014012.nc'
fpaths_hus_A4['mp'] = folder+'hus_Amon_MPI-ESM-LR_abrupt4xCO2_r1i1p1_185001-189912.nc'
fpaths_hus_A4['no'] = folder+'hus_Amon_NorESM1-M_abrupt4xCO2_r1i1p1_000101-010012.nc'


#Wheter to load from a file (so I don't have to take zonal mean multiple times)

loadNpyFile_hus_PI = OrderedDict()
loadNpyFile_hus_PI['bn'] = False
loadNpyFile_hus_PI['ca'] = False
loadNpyFile_hus_PI['cc'] = False
loadNpyFile_hus_PI['ce'] = False
loadNpyFile_hus_PI['cs'] = False
loadNpyFile_hus_PI['gi'] = False
loadNpyFile_hus_PI['hg'] = False
loadNpyFile_hus_PI['ip'] = False
loadNpyFile_hus_PI['mi'] = False
loadNpyFile_hus_PI['mp'] = False
loadNpyFile_hus_PI['no'] = False

loadNpyFile_hus_G1 = OrderedDict()
loadNpyFile_hus_G1['bn'] = False
loadNpyFile_hus_G1['ca'] = False
loadNpyFile_hus_G1['cc'] = False
loadNpyFile_hus_G1['ce'] = False
loadNpyFile_hus_G1['cs'] = False
loadNpyFile_hus_G1['gi'] = False
loadNpyFile_hus_G1['hg'] = False
loadNpyFile_hus_G1['ip'] = False
loadNpyFile_hus_G1['mi'] = False 
loadNpyFile_hus_G1['mp'] = False
loadNpyFile_hus_G1['no'] = False

loadNpyFile_hus_A4 = OrderedDict()
loadNpyFile_hus_A4['bn'] = False
loadNpyFile_hus_A4['ca'] = False
loadNpyFile_hus_A4['cc'] = False
loadNpyFile_hus_A4['ce'] = False
loadNpyFile_hus_A4['cs'] = False
loadNpyFile_hus_A4['gi'] = False
loadNpyFile_hus_A4['hg'] = False
loadNpyFile_hus_A4['ip'] = False
loadNpyFile_hus_A4['mi'] = False
loadNpyFile_hus_A4['mp'] = False
loadNpyFile_hus_A4['no'] = False



#Load NetCDF files & create NetCDF Dataset objects
Datasets_hus_PI = OrderedDict()    
for key, value in fpaths_hus_PI.items():
    print('Loading NetCDF file (hus): piControl: ' + gmf.modelNames[key]) #Debug
    Datasets_hus_PI[key] = nc4.Dataset(value)    

Datasets_hus_G1 = OrderedDict()
for key, value in fpaths_hus_G1.items():
    print('Loading NetCDF file (hus): G1: ' + gmf.modelNames[key]) #Debug
    Datasets_hus_G1[key] = nc4.Dataset(value)
    
Datasets_hus_A4 = OrderedDict()    
for key, value in fpaths_hus_A4.items():
    print('Loading NetCDF file (hus): abrupt4xCO2: ' + gmf.modelNames[key]) #Debug
    Datasets_hus_A4[key] = nc4.Dataset(value)    



#Calculate zonal means, averaged over 40 years
#For G1 and Abrupt4xCO2, the average was over years 11-50 in the Kravitz et al. papers; (10-49 in Python indexing)
#For piControl, it doesn't matter which 40 years, but use 1st 40 years for consistency.


#The standard pressure levels from CMIP5: (hPa)     
levs = np.array([1000, 925, 850, 700, 600, 500, 400, 300, 250, 200, 150, 100, 70, 50, 30, 20, 10]) 
npyFolder = 'npyFiles/' #Locations of the npy files


#Do the pressure interpolation and zonal mean calculation and save to npy file, or load from existing npy files


#piControl: years 0-39 (doesn't matter which 40 years in this case)        
zonalMeans_hus_PI = OrderedDict()
for key, value in Datasets_hus_PI.items():
    if not(loadNpyFile_hus_PI[key]):
        print('Calculating zonal mean specific humidity: piControl: ' + gmf.modelNames[key])
        zonalMeans_hus_PI[key] = gmf.zonalMeanMultiYearPressureCoordinates(value, 'hus', 0, 39)        
        #Subset to the pressure levels of interest
        if key == 'mi': #This model reported extra layers in between the standard CESM ones--listed at bottom
            zonalMeans_hus_PI[key] = np.concatenate((zonalMeans_hus_PI[key][0:10,:],
                                                    zonalMeans_hus_PI[key][11:12,:],
                                                    zonalMeans_hus_PI[key][14:15,:],
                                                    zonalMeans_hus_PI[key][17:21,:],
                                                    zonalMeans_hus_PI[key][22:23,:]),    axis=0)
        else:
            zonalMeans_hus_PI[key] = zonalMeans_hus_PI[key][0:17,:] #Get rid of any pressure levels beyond 10 hPa
        if not(key in ['cc', 'cs', 'gi', 'hg', 'ip', 'mi', 'ce']): #Maksed arrays--use dump
            np.save(npyFolder+'zonalMeans_hus_PI_'+key+'.npy', zonalMeans_hus_PI[key])
        else:
            zonalMeans_hus_PI[key].dump(npyFolder+'zonalMeans_hus_PI_'+key+'.npy')
    else:
        print('Loading zonal mean specific humidity from npy file: piControl: ' + gmf.modelNames[key])
        zonalMeans_hus_PI[key] = np.load(npyFolder+'zonalMeans_hus_PI_'+key+'.npy')        


#G1: years 10-49 (if year indices start from 0)
zonalMeans_hus_G1 = OrderedDict()
for key, value in Datasets_hus_G1.items():
    if not(loadNpyFile_hus_G1[key]):
        print('Calculating zonal mean specific humidity: G1: ' + gmf.modelNames[key])
        zonalMeans_hus_G1[key] = gmf.zonalMeanMultiYearPressureCoordinates(value, 'hus', 10, 49)
        #Subset to the pressure levels of interest
        if key == 'mi': #This model reported extra layers in between the standard CESM ones--listed at bottom
            zonalMeans_hus_G1[key] = np.concatenate((zonalMeans_hus_G1[key][0:10,:],
                                                    zonalMeans_hus_G1[key][11:12,:],
                                                    zonalMeans_hus_G1[key][14:15,:],
                                                    zonalMeans_hus_G1[key][17:21,:],
                                                    zonalMeans_hus_G1[key][22:23,:]),    axis=0)
        else:
            zonalMeans_hus_G1[key] = zonalMeans_hus_G1[key][0:17,:] #Get rid of any pressure levels beyond 10 hPa
        if not(key in ['cc', 'cs', 'gi', 'hg', 'ip', 'mi', 'ce']): #These models have masked arrays which...
            np.save(npyFolder+'zonalMeans_hus_G1_'+key+'.npy', zonalMeans_hus_G1[key]) #...can't be saved using np.save
        else:
            zonalMeans_hus_G1[key].dump(npyFolder+'zonalMeans_hus_G1_'+key+'.npy')
    else:
        print('Loading zonal mean specific humidity from npy file: G1: ' + gmf.modelNames[key])
        zonalMeans_hus_G1[key] = np.load(npyFolder+'zonalMeans_hus_G1_'+key+'.npy')


#abrupt4xCO2: years 10-49        
zonalMeans_hus_A4 = OrderedDict()
for key, value in Datasets_hus_A4.items():
    if not(loadNpyFile_hus_A4[key]):
        print('Calculating zonal mean specific humidity: abrupt4xCO2: ' + gmf.modelNames[key])
        zonalMeans_hus_A4[key] = gmf.zonalMeanMultiYearPressureCoordinates(value, 'hus', 10, 49)
        #Subset to the pressure levels of interest
        if key == 'mi': #This model reported extra layers in between the standard CESM ones--listed at bottom
            zonalMeans_hus_A4[key] = np.concatenate((zonalMeans_hus_A4[key][0:10,:],
                                                    zonalMeans_hus_A4[key][11:12,:],
                                                    zonalMeans_hus_A4[key][14:15,:],
                                                    zonalMeans_hus_A4[key][17:21,:],
                                                    zonalMeans_hus_A4[key][22:23,:]),    axis=0)
        else:
            zonalMeans_hus_A4[key] = zonalMeans_hus_A4[key][0:17,:] #Get rid of any pressure levels beyond 10 hPa
        if not(key in ['cc', 'cs', 'gi', 'hg', 'ip', 'mi', 'ce']): #Masked arrays--use dump
            np.save(npyFolder+'zonalMeans_hus_A4_'+key+'.npy', zonalMeans_hus_A4[key])
        else:
            zonalMeans_hus_A4[key].dump(npyFolder+'zonalMeans_hus_A4_'+key+'.npy')
    else:
        print('Loading zonal mean specific humidity from npy file: abrupt4xCO2: ' + gmf.modelNames[key])
        zonalMeans_hus_A4[key] = np.load(npyFolder+'zonalMeans_hus_A4_'+key+'.npy')        

print('Finished zonal mean calculations or loading.')






#####   PLOTS   #####


#For ACP paper: subplot version

keys = ['bn', 'ca', 'cc', 'ce', 'cs', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']
(w,h) = plt.figaspect(0.7)*1.9
fig = plt.figure(500, figsize=(w,h))
for i in range(11):
    plt.subplot(3,4,i+1)
    lat = Datasets_hus_G1[keys[i]].variables['lat'][:]
    plotData = np.log(zonalMeans_hus_G1[keys[i]]) - np.log(zonalMeans_hus_PI[keys[i]])
    thePlot = plt.contourf(np.sin(lat*np.pi/180.), levs, plotData, levels=np.linspace(-.25,.25,12), cmap=plt.cm.BrBG, extend='both')
    plt.gca().invert_yaxis()
    #if i in range(7,10):
    if i in range(8,11):
        plt.xlabel('Latitude ($^\circ$N)', fontsize=15)
    plt.xticks(np.sin(np.array([-90, -30, 0, 30, 90])*np.pi/180.),  ['-90', '-30', '0', '30', '90'])
    if i in [0,4,8]:
        plt.ylabel('Pressure (hPa)', fontsize=15) #Y axis labels running alon the left side
    plt.yticks([1000, 800, 600, 400, 200, 0])
    plt.title(gmf.modelNames[keys[i]], fontsize=15)
    plt.gca().set_axis_bgcolor('k') 
#Colorbar
cbar = plt.colorbar(thePlot, cax=fig.add_axes([0.78, 0.18, 0.2, 0.03]), ticks=np.linspace(-.25, .25, 5), orientation='horizontal')
cbar.ax.set_xlabel('Humidity change (log kg/kg)', fontsize=15)
cbar.ax.tick_params(labelsize=15)
cbar.ax.set_xticklabels(['-.25', '-.125', '0', '.125', '.25'])
plt.tight_layout()
plt.savefig('plots/log_hus_zonalMean_diff_ACP_paperVersion.pdf')

         
plt.show()
         
        

