# Script to plot maps of possible predictors of low cloud changes in GeoMIP experiments, including EIS, LTS
#
# Rick Russotto
# Started 7-5-17


import numpy as np
import matplotlib.pyplot as plt
import netCDF4 as nc4
import geomipFunctions as gmf
from mpl_toolkits.basemap import Basemap
import os
from collections import OrderedDict


# For which models do I have all the necessary output? (tas, ta, huss, ps)
# Of the 9 models in "cloudFractionMaps", all but MPI-ESM, which only had 
# hus instead of huss. Would have to use lowest model layer, which on average
# is much higher than 2 meters up. Would also need to modify function code to allow for this. 

# Paths for the files, in dictionaries, for EIS calculations
folder = '../nobackup/' 

paths_EIS_PI = OrderedDict()
paths_EIS_PI['bn'] = {'tas':  folder+'tas_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc',
                      'ta':   folder+'ta_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc',
                      'huss': folder+'huss_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc',
                      'ps':   folder+'ps_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc'
                      }
paths_EIS_PI['ca'] = {'tas':  folder+'tas_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc',
                      'ta':   folder+'ta_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc',
                      'huss': folder+'huss_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc',
                      'ps':   folder+'ps_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc'
                      }
paths_EIS_PI['cc'] = {'tas':  folder+'tas_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc',
                      'ta':   folder+'ta_Amon_CCSM4_piControl_r1i1p1_080001-084912.nc',
                      'huss': folder+'huss_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc',
                      'ps':   folder+'ps_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc'
                      }
paths_EIS_PI['ce'] = {'tas':  folder+'tas_Amon_CESM-CAM5.1-FV_piControl_r1i1p1_000101-005012.nc',
                      'ta':   folder+'ta_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc',
                      'huss': folder+'huss_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc',
                      'ps':   folder+'ps_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc'
                      }
paths_EIS_PI['gi'] = {'tas':  folder+'tas_Amon_GISS-E2-R_piControl_r1i1p1_185001-191912.nc',
                      'ta':   folder+'ta_Amon_GISS-E2-R_piControl_r1i1p1_185001-191912.nc',
                      'huss': folder+'huss_Amon_GISS-E2-R_piControl_r1i1p1_185001-191912.nc',
                      'ps':   folder+'ps_Amon_GISS-E2-R_piControl_r1i1p1_185001-191912.nc'
                      }
paths_EIS_PI['hg'] = {'tas':  folder+'tas_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc',
                      'ta':   folder+'ta_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc',
                      'huss': folder+'huss_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc',
                      'ps':   folder+'ps_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc'
                      }
paths_EIS_PI['ip'] = {'tas':  folder+'tas_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-279912.nc',
                      'ta':   folder+'ta_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-189912.nc',
                      'huss': folder+'huss_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-279912.nc',
                      'ps':   folder+'ps_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-279912.nc'
                      }
paths_EIS_PI['mi'] = {'tas':  folder+'tas_Amon_MIROC-ESM_piControl_r1i1p1_180001-233012.nc',
                      'ta':   folder+'ta_Amon_MIROC-ESM_piControl_r1i1p1_180001-193912.nc',
                      'huss': folder+'huss_Amon_MIROC-ESM_piControl_r1i1p1_180001-233012.nc',
                      'ps':   folder+'ps_Amon_MIROC-ESM_piControl_r1i1p1_180001-233012.nc'
                      } 
paths_EIS_PI['no'] = {'tas':  folder+'tas_Amon_NorESM1-M_piControl_r1i1p1_070101-115012.nc',
                      'ta':   folder+'ta_Amon_NorESM1-M_piControl_r1i1p1_070101-079912.nc', 
                      'huss': folder+'huss_Amon_NorESM1-M_piControl_r1i1p1_070101-115012.nc',
                      'ps':   folder+'ps_Amon_NorESM1-M_piControl_r1i1p1_070101-115012.nc'
                      }                  
                  
                  
paths_EIS_G1 = OrderedDict()
#paths_EIS_G1['bn'] = {'tas':  folder+'tas_Amon_BNU-ESM_G1_r1i1p1_185001-190012.nc',
#                      'ta':   folder+'ta_Amon_BNU-ESM_G1_r1i1p1_185001-190012.nc',
#                      'huss': folder+'huss_Amon_BNU-ESM_G1_r1i1p1_185001-190012.nc',
#                      'ps':   folder+'ps_Amon_BNU-ESM_G1_r1i1p1_185001-190012.nc'
#                      }
paths_EIS_G1['bn'] = {'tas':  folder+'tas_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc',
                      'ta':   folder+'ta_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc',
                      'huss':  folder+'huss_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc',
                      'ps':   folder+'ps_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc'
                      }
paths_EIS_G1['ca'] = {'tas':  folder+'tas_Amon_CanESM2_G1_r1i1p1_185001-194912.nc',
                      'ta':   folder+'ta_Amon_CanESM2_G1_r1i1p1_185001-194912.nc',
                      'huss': folder+'huss_Amon_CanESM2_G1_r1i1p1_185001-194912.nc',
                      'ps':   folder+'ps_Amon_CanESM2_G1_r1i1p1_185001-194912.nc'
                      }
paths_EIS_G1['cc'] = {'tas':  folder+'tas_Amon_CCSM4_G1_r1i1p1_185001-192412.nc',
                      'ta':   folder+'ta_Amon_CCSM4_G1_r1i1p1_185001-192412.nc',
                      'huss': folder+'huss_Amon_CCSM4_G1_r1i1p1_185001-192412.nc',
                      'ps':   folder+'ps_Amon_CCSM4_G1_r1i1p1_185001-192412.nc'
                      }
paths_EIS_G1['ce'] = {'tas':  folder+'tas_Amon_CESM-CAM5.1-FV_G1_r1i1p1_000101-005912.nc',
                      'ta':   folder+'ta_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc',
                      'huss': folder+'huss_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc',
                      'ps':   folder+'ps_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc'
                      }
paths_EIS_G1['gi'] = {'tas':  folder+'tas_Amon_GISS-E2-R_G1_r1i1p1_185001-191912.nc',
                      'ta':   folder+'ta_Amon_GISS-E2-R_G1_r1i1p1_185001-191912.nc',
                      'huss': folder+'huss_Amon_GISS-E2-R_G1_r1i1p1_185001-191912.nc',
                      'ps':   folder+'ps_Amon_GISS-E2-R_G1_r1i1p1_185001-191912.nc'
                      }
paths_EIS_G1['hg'] = {'tas':  folder+'tas_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc',
                      'ta':   folder+'ta_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc',
                      'huss': folder+'huss_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc',
                      'ps':   folder+'ps_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc'
                      }                      
paths_EIS_G1['ip'] = {'tas':  folder+'tas_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc',
                      'ta':   folder+'ta_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc',
                      'huss': folder+'huss_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc',
                      'ps':   folder+'ps_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc'
                      }
paths_EIS_G1['mi'] = {'tas':  folder+'tas_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc',
                      'ta':   folder+'ta_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc',
                      'huss': folder+'huss_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc',
                      'ps':   folder+'ps_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc'
                      }
paths_EIS_G1['no'] = {'tas':  folder+'tas_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc',
                      'ta':   folder+'ta_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc',
                      'huss': folder+'huss_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc',
                      'ps':   folder+'ps_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc'
                      }           

paths_EIS_A4 = OrderedDict()
paths_EIS_A4['bn'] = {'tas':  folder+'tas_Amon_BNU-ESM_abrupt4xCO2_r1i1p1_185001-199912.nc',
                      'ta':   folder+'ta_Amon_BNU-ESM_abrupt4xCO2_r1i1p1_185001-199912.nc',
                      'huss': folder+'huss_Amon_BNU-ESM_abrupt4xCO2_r1i1p1_185001-199912.nc',
                      'ps':   folder+'ps_Amon_BNU-ESM_abrupt4xCO2_r1i1p1_185001-199912.nc'
                      }
paths_EIS_A4['ca'] = {'tas':  folder+'tas_Amon_CanESM2_abrupt4xCO2_r1i1p1_185001-199912.nc',
                      'ta':   folder+'ta_Amon_CanESM2_abrupt4xCO2_r1i1p1_185001-199912.nc',
                      'huss': folder+'huss_Amon_CanESM2_abrupt4xCO2_r1i1p1_185001-199912.nc',
                      'ps':   folder+'ps_Amon_CanESM2_abrupt4xCO2_r1i1p1_185001-199912.nc'
                      }
paths_EIS_A4['cc'] = {'tas':  folder+'tas_Amon_CCSM4_abrupt4xCO2_r1i1p1_185001-200012.nc',
                      'ta':   folder+'ta_Amon_CCSM4_abrupt4xCO2_r1i1p1_185001-189912.nc',
                      'huss': folder+'huss_Amon_CCSM4_abrupt4xCO2_r1i1p1_185001-200012.nc',
                      'ps':   folder+'ps_Amon_CCSM4_abrupt4xCO2_r1i1p1_185001-200012.nc'
                      }
paths_EIS_A4['ce'] = {'tas':  folder+'tas_Amon_CESM1-CAM5-1-FV2_abrupt4xCO2_r3i1p1_000101-015012.nc',
                      'ta':   folder+'ta_Amon_CESM1-CAM5-1-FV2_abrupt4xCO2_r3i1p1_000101-015012.nc',
                      'huss': folder+'huss_Amon_CESM1-CAM5-1-FV2_abrupt4xCO2_r3i1p1_000101-015012.nc',
                      'ps':   folder+'ps_Amon_CESM1-CAM5-1-FV2_abrupt4xCO2_r3i1p1_000101-015012.nc'
                      }
paths_EIS_A4['gi'] = {'tas':  folder+'tas_Amon_GISS-E2-R_abrupt4xCO2_r1i1p1_185001-191912.nc',
                      'ta':   folder+'ta_Amon_GISS-E2-R_abrupt4xCO2_r1i1p1_185001-191912.nc',
                      'huss': folder+'huss_Amon_GISS-E2-R_abrupt4xCO2_r1i1p1_185001-191912.nc',
                      'ps':   folder+'ps_Amon_GISS-E2-R_abrupt4xCO2_r1i1p1_185001-191912.nc'
                      }
paths_EIS_A4['hg'] = {'tas':  folder+'tas_Amon_HadGEM2-ES_abrupt4xCO2_r1i1p1_185912-191011.nc',
                      'ta':   folder+'ta_Amon_HadGEM2-ES_abrupt4xCO2_r1i1p1_185912-191011.nc',
                      'huss': folder+'huss_Amon_HadGEM2-ES_abrupt4xCO2_r1i1p1_185912-191011.nc',
                      'ps':   folder+'ps_Amon_HadGEM2-ES_abrupt4xCO2_r1i1p1_185912-190911.nc'
                      }                      
paths_EIS_A4['ip'] = {'tas':  folder+'tas_Amon_IPSL-CM5A-LR_abrupt4xCO2_r1i1p1_185001-204912.nc',
                      'ta':   folder+'ta_Amon_IPSL-CM5A-LR_abrupt4xCO2_r1i1p1_185001-194912.nc',
                      'huss': folder+'huss_Amon_IPSL-CM5A-LR_abrupt4xCO2_r1i1p1_185001-204912.nc',
                      'ps':   folder+'ps_Amon_IPSL-CM5A-LR_abrupt4xCO2_r1i1p1_185001-204912.nc'
                      }
paths_EIS_A4['mi'] = {'tas':  folder+'tas_Amon_MIROC-ESM_abrupt4xCO2_r1i1p1_000101-015012.nc',
                      'ta':   folder+'ta_Amon_MIROC-ESM_abrupt4xCO2_r1i1p1_000101-014012.nc',
                      'huss': folder+'huss_Amon_MIROC-ESM_abrupt4xCO2_r1i1p1_000101-015012.nc',
                      'ps':   folder+'ps_Amon_MIROC-ESM_abrupt4xCO2_r1i1p1_000101-015012.nc'
                      }
paths_EIS_A4['no'] = {'tas':  folder+'tas_Amon_NorESM1-M_abrupt4xCO2_r1i1p1_000101-015012.nc',
                      'ta':   folder+'ta_Amon_NorESM1-M_abrupt4xCO2_r1i1p1_000101-010012.nc',
                      'huss': folder+'huss_Amon_NorESM1-M_abrupt4xCO2_r1i1p1_000101-015012.nc',
                      'ps':   folder+'ps_Amon_NorESM1-M_abrupt4xCO2_r1i1p1_000101-015012.nc'
                      }                             
                  
                  
#Create NetCDF Dataset objects: nested dicts, like in "v2_meridionalEnergyTransport.py"
#("key" refers to model, "innerKey" refers to variable)               


Datasets_EIS_PI = OrderedDict()
for key in paths_EIS_PI.keys():
    Datasets_EIS_PI[key] = dict()
    for innerKey in paths_EIS_PI[key].keys():
        Datasets_EIS_PI[key][innerKey] = nc4.Dataset(paths_EIS_PI[key][innerKey])                       
                  
Datasets_EIS_G1 = OrderedDict()
for key in paths_EIS_G1.keys():
    Datasets_EIS_G1[key] = dict()
    for innerKey in paths_EIS_G1[key].keys():
        Datasets_EIS_G1[key][innerKey] = nc4.Dataset(paths_EIS_G1[key][innerKey])

Datasets_EIS_A4 = OrderedDict()
for key in paths_EIS_A4.keys():
    Datasets_EIS_A4[key] = dict()
    for innerKey in paths_EIS_A4[key].keys():
        Datasets_EIS_A4[key][innerKey] = nc4.Dataset(paths_EIS_A4[key][innerKey])  
        
# For HadGEM2-ES model, need to interpolate the surface variables to the "ta" 
# grid which was reported on the midpoint latitudes.
# to the "tas" grid on which all the other variables are plotted. 
Datasets_EIS_PI['hg']['tas'].variables['tas'] = (Datasets_EIS_PI['hg']['tas'].variables['tas'][:,1:,:] + Datasets_EIS_PI['hg']['tas'].variables['tas'][:,:-1,:])/2.
Datasets_EIS_PI['hg']['huss'].variables['huss'] = (Datasets_EIS_PI['hg']['huss'].variables['huss'][:,1:,:] + Datasets_EIS_PI['hg']['huss'].variables['huss'][:,:-1,:])/2.
Datasets_EIS_PI['hg']['ps'].variables['ps'] = (Datasets_EIS_PI['hg']['ps'].variables['ps'][:,1:,:] + Datasets_EIS_PI['hg']['ps'].variables['ps'][:,:-1,:])/2.

Datasets_EIS_G1['hg']['tas'].variables['tas'] = (Datasets_EIS_G1['hg']['tas'].variables['tas'][:,1:,:] + Datasets_EIS_G1['hg']['tas'].variables['tas'][:,:-1,:])/2.
Datasets_EIS_G1['hg']['huss'].variables['huss'] = (Datasets_EIS_G1['hg']['huss'].variables['huss'][:,1:,:] + Datasets_EIS_G1['hg']['huss'].variables['huss'][:,:-1,:])/2.
Datasets_EIS_G1['hg']['ps'].variables['ps'] = (Datasets_EIS_G1['hg']['ps'].variables['ps'][:,1:,:] + Datasets_EIS_G1['hg']['ps'].variables['ps'][:,:-1,:])/2.

Datasets_EIS_A4['hg']['tas'].variables['tas'] = (Datasets_EIS_A4['hg']['tas'].variables['tas'][:,1:,:] + Datasets_EIS_A4['hg']['tas'].variables['tas'][:,:-1,:])/2.
Datasets_EIS_A4['hg']['huss'].variables['huss'] = (Datasets_EIS_A4['hg']['huss'].variables['huss'][:,1:,:] + Datasets_EIS_A4['hg']['huss'].variables['huss'][:,:-1,:])/2.
Datasets_EIS_A4['hg']['ps'].variables['ps'] = (Datasets_EIS_A4['hg']['ps'].variables['ps'][:,1:,:] + Datasets_EIS_A4['hg']['ps'].variables['ps'][:,:-1,:])/2.


#Calculate EIS (save to npy files)

loadFromFile_EIS_PI ={'bn': False, 
                      'ca': False, 
                      'cc': False,
                      'ce': False, 
                      'gi': False,
                      'hg': False,
                      'ip': False, 
                      'mi': False, 
                      'no': False}

loadFromFile_EIS_G1 ={'bn': False,
                      'ca': False, 
                      'cc': False,
                      'ce': False, 
                      'gi': False,
                      'hg': False,
                      'ip': False, 
                      'mi': False, 
                      'no': False}
                     
loadFromFile_EIS_A4 ={'bn': False, 
                      'ca': False, 
                      'cc': False,
                      'ce': False, 
                      'gi': False,
                      'hg': False,
                      'ip': False, 
                      'mi': False, 
                      'no': False}

models_EIS = ['bn', 'ca', 'cc', 'ce', 'gi', 'hg', 'ip', 'mi', 'no'] #Getting overflow warnings for CCSM4, 

#Index of 700 hPa pressure level for each model: (starting from 0) (checked using ncdump)
#bn: 3
#ca: 3
#cc: 3
#ce: 3
#gi: 3 (has extra levels beyond 1000 Pa, but this doesn't matter because they're in order of descending pressure)
#hg: 3
#ip: 3
#mi: 3 (has some extra pressure levels, but none between 1000 and 700 hPa)
#no: 3

#Okay, if 3 is the standard index for the 700 hPa pressure level, make this the default for the function. 
#But specifiy it explicitly here too, just to be safe.

EIS_PI = dict()
for key in models_EIS:
    if not loadFromFile_EIS_PI[key]:
        print('Calculating EIS: piControl: ' + gmf.modelNames[key])
        EIS_PI[key] = gmf.multiYearMeanEIS(Datasets_EIS_PI[key], 0, 39, index700=3)
        if not(key in ['cc', 'ce', 'gi', 'hg', 'ip', 'mi']): #These models have masked arrays which...
            np.save('npyFiles/EIS_PI_'+key+'.npy', EIS_PI[key]) #...can't be saved using np.save
        else:
            EIS_PI[key].dump('npyFiles/EIS_PI_'+key+'.npy')
    else:
        print('Loading EIS: piControl: ' + gmf.modelNames[key])
        EIS_PI[key] = np.load('npyFiles/EIS_PI_'+key+'.npy')
        
EIS_G1 = dict()
for key in models_EIS:
    if not loadFromFile_EIS_G1[key]:
        print('Calculating EIS: G1: ' + gmf.modelNames[key])
        EIS_G1[key] = gmf.multiYearMeanEIS(Datasets_EIS_G1[key], 10, 49, index700=3)
        if not(key in ['cc', 'ce', 'gi', 'hg', 'ip', 'mi']): #These models have masked arrays which...
            np.save('npyFiles/EIS_G1_'+key+'.npy', EIS_G1[key]) #...can't be saved using np.save
        else:
            EIS_G1[key].dump('npyFiles/EIS_G1_'+key+'.npy')
    else:
        print('Loading EIS: G1: ' + gmf.modelNames[key])
        EIS_G1[key] = np.load('npyFiles/EIS_G1_'+key+'.npy')

EIS_A4 = dict()
for key in models_EIS:
    if not loadFromFile_EIS_A4[key]:
        print('Calculating EIS: abrupt4xCO2: ' + gmf.modelNames[key])
        EIS_A4[key] = gmf.multiYearMeanEIS(Datasets_EIS_A4[key], 10, 49, index700=3)
        if not(key in ['cc', 'ce', 'gi', 'hg', 'ip', 'mi']): #These models have masked arrays which...
            np.save('npyFiles/EIS_A4_'+key+'.npy', EIS_A4[key]) #...can't be saved using np.save
        else:
            EIS_A4[key].dump('npyFiles/EIS_A4_'+key+'.npy')
    else:
        print('Loading EIS: abrupt4xCO2: ' + gmf.modelNames[key])
        EIS_A4[key] = np.load('npyFiles/EIS_A4_'+key+'.npy')        
        
        

#Other calculations 

#LTS
loadFromFile_LTS_PI ={'bn': False, 
                      'ca': False, 
                      'cc': False,
                      'ce': False, 
                      'gi': False,
                      'hg': False,
                      'ip': False, 
                      'mi': False, 
                      'no': False}

loadFromFile_LTS_G1 ={'bn': False, 
                      'ca': False, 
                      'cc': False,
                      'ce': False, 
                      'gi': False,
                      'hg': False,
                      'ip': False, 
                      'mi': False, 
                      'no': False}
                     
loadFromFile_LTS_A4 ={'bn': False, 
                      'ca': False, 
                      'cc': False,
                      'ce': False, 
                      'gi': False,
                      'hg': False,
                      'ip': False, 
                      'mi': False, 
                      'no': False}

models_LTS = ['bn', 'ca', 'cc', 'ce', 'gi', 'hg', 'ip', 'mi', 'no'] 

LTS_PI = dict()
for key in models_LTS:
    if not loadFromFile_LTS_PI[key]:
        print('Calculating LTS: piControl: ' + gmf.modelNames[key])
        LTS_PI[key] = gmf.multiYearMeanLTS(Datasets_EIS_PI[key], 0, 39, index700=3)
        if not(key in ['cc', 'ce', 'gi', 'hg', 'ip', 'mi']): #These models have masked arrays which...
            np.save('npyFiles/LTS_PI_'+key+'.npy', LTS_PI[key]) #...can't be saved using np.save
        else:
            LTS_PI[key].dump('npyFiles/LTS_PI_'+key+'.npy')
    else:
        print('Loading LTS: piControl: ' + gmf.modelNames[key])
        LTS_PI[key] = np.load('npyFiles/LTS_PI_'+key+'.npy')
        
LTS_G1 = dict()
for key in models_LTS:
    if not loadFromFile_LTS_G1[key]:
        print('Calculating LTS: G1: ' + gmf.modelNames[key])
        LTS_G1[key] = gmf.multiYearMeanLTS(Datasets_EIS_G1[key], 10, 49, index700=3)
        if not(key in ['cc', 'ce', 'gi', 'hg', 'ip', 'mi']): #These models have masked arrays which...
            np.save('npyFiles/LTS_G1_'+key+'.npy', LTS_G1[key]) #...can't be saved using np.save
        else:
            LTS_G1[key].dump('npyFiles/LTS_G1_'+key+'.npy')
    else:
        print('Loading LTS: G1: ' + gmf.modelNames[key])
        LTS_G1[key] = np.load('npyFiles/LTS_G1_'+key+'.npy')

LTS_A4 = dict()
for key in models_LTS:
    if not loadFromFile_LTS_A4[key]:
        print('Calculating LTS: abrupt4xCO2: ' + gmf.modelNames[key])
        LTS_A4[key] = gmf.multiYearMeanLTS(Datasets_EIS_A4[key], 10, 49, index700=3)
        if not(key in ['cc', 'ce', 'gi', 'hg', 'ip', 'mi']): #These models have masked arrays which...
            np.save('npyFiles/LTS_A4_'+key+'.npy', LTS_A4[key]) #...can't be saved using np.save
        else:
            LTS_A4[key].dump('npyFiles/LTS_A4_'+key+'.npy')
    else:
        print('Loading LTS: abrupt4xCO2: ' + gmf.modelNames[key])
        LTS_A4[key] = np.load('npyFiles/LTS_A4_'+key+'.npy')       
        
        



#####   PLOTS   #####

#Difference maps for EIS

#Version that saves PNG plot, with smaller color bar and bigger text
def mapDiffContourf_PNG(figNo, key, run1, run2, clevs=np.linspace(-2, 2, 16),  save=False):
    w,h = plt.figaspect(1./2.5)  
    plt.figure(figNo, figsize=(w,h))     
    if(run1 == 'G1' and run2 == 'PI'):
        lat = Datasets_EIS_G1[key]['ta'].variables['lat'][:]
        lon = Datasets_EIS_G1[key]['ta'].variables['lon'][:]
        plotData = EIS_G1[key] - EIS_PI[key]
    elif(run1 == 'A4' and run2 == 'PI'):
        lat = Datasets_EIS_A4[key]['ta'].variables['lat'][:]
        lon = Datasets_EIS_A4[key]['ta'].variables['lon'][:]
        plotData = EIS_A4[key] - EIS_PI[key]
    else:
        print('Haven\'t programmed this combination of runs yet; exiting plot difference function')
        return
    #Duplicate edge data to eliminate white strip on left side
    titleString = gmf.modelNames[key]
    lon = np.append(lon,360.) 
    plotData = np.append(plotData, plotData[:,0:1], axis=1)  
    #Make map
    Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
    Map.drawcoastlines(linewidth=1)
    Map.drawparallels(np.linspace(-90.,90.,7))
    Map.drawmeridians(np.linspace(-150., 150., 11))
    lons, lats = np.meshgrid(lon, lat)
    #thePlot = Map.contourf(lons, lats, plotData, latlon=True, cmap=plt.cm.RdBu, levels=clevs, extend='both')
    thePlot = Map.contourf(lons, lats, plotData, latlon=True, cmap=plt.cm.RdBu_r, levels=clevs, extend='both')
    cbar = plt.colorbar(thePlot)
    cbar.ax.set_ylabel('EIS difference (K)', fontsize=24)
    cbar.ax.tick_params(labelsize=22)
    plt.title(titleString, fontsize=26)
    Map.fillcontinents() #Block out land -- these quantities are only valid over the ocean
    plt.tight_layout()
    if(save):   
        #plt.savefig('plots/individualModels/EIS_map_'+run1+'-'+run2+'_'+key+'.png') 
        plt.savefig('plots/individualModels/EIS_map_'+run1+'-'+run2+'_'+key+'_flipped.png') 

#Plots for G1 - piControl
mapDiffContourf_PNG(510, 'bn', 'G1', 'PI', save=True)
mapDiffContourf_PNG(511, 'ca', 'G1', 'PI', save=True)
mapDiffContourf_PNG(512, 'cc', 'G1', 'PI', save=True)
mapDiffContourf_PNG(513, 'ce', 'G1', 'PI', save=True)
mapDiffContourf_PNG(514, 'gi', 'G1', 'PI', save=True)
mapDiffContourf_PNG(515, 'hg', 'G1', 'PI', save=True)
mapDiffContourf_PNG(516, 'ip', 'G1', 'PI', save=True)
mapDiffContourf_PNG(517, 'mi', 'G1', 'PI', save=True)
mapDiffContourf_PNG(518, 'no', 'G1', 'PI', save=True)



#LTS maps


#PNG version
def mapDiffContourfLTS_PNG(figNo, key, run1, run2, clevs=np.linspace(-2, 2, 16),  save=False):
    w,h = plt.figaspect(1./2.5)  
    plt.figure(figNo, figsize=(w,h))     
    if(run1 == 'G1' and run2 == 'PI'):
        lat = Datasets_EIS_G1[key]['ta'].variables['lat'][:]
        lon = Datasets_EIS_G1[key]['ta'].variables['lon'][:]
        plotData = LTS_G1[key] - LTS_PI[key]
    elif(run1 == 'A4' and run2 == 'PI'):
        lat = Datasets_EIS_A4[key]['ta'].variables['lat'][:]
        lon = Datasets_EIS_A4[key]['ta'].variables['lon'][:]
        plotData = LTS_A4[key] - LTS_PI[key]
    else:
        print('Haven\'t programmed this combination of runs yet; exiting plot difference function')
        return
    #Duplicate edge data to eliminate white strip on left side
    titleString = gmf.modelNames[key]
    lon = np.append(lon,360.) 
    plotData = np.append(plotData, plotData[:,0:1], axis=1)  
    #Make map
    Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
    Map.drawcoastlines(linewidth=1)
    Map.drawparallels(np.linspace(-90.,90.,7))
    Map.drawmeridians(np.linspace(-150., 150., 11))
    lons, lats = np.meshgrid(lon, lat)
    #thePlot = Map.contourf(lons, lats, plotData, latlon=True, cmap=plt.cm.RdBu, levels=clevs, extend='both')
    thePlot = Map.contourf(lons, lats, plotData, latlon=True, cmap=plt.cm.RdBu_r, levels=clevs, extend='both')
    cbar = plt.colorbar(thePlot)
    cbar.ax.set_ylabel('LTS difference (K)', fontsize=24)
    cbar.ax.tick_params(labelsize=22)
    plt.title(titleString, fontsize=26)
    Map.fillcontinents() #Block out land -- these quantities are only valid over the ocean
    plt.tight_layout()
    if(save):   
        #plt.savefig('plots/individualModels/LTS_map_'+run1+'-'+run2+'_'+key+'.png') 
        plt.savefig('plots/individualModels/LTS_map_'+run1+'-'+run2+'_'+key+'_flipped.png') 

#Plots for G1 - piControl
mapDiffContourfLTS_PNG(640, 'bn', 'G1', 'PI', save=True)
mapDiffContourfLTS_PNG(641, 'ca', 'G1', 'PI', save=True)
mapDiffContourfLTS_PNG(642, 'cc', 'G1', 'PI', save=True)
mapDiffContourfLTS_PNG(643, 'ce', 'G1', 'PI', save=True)
mapDiffContourfLTS_PNG(644, 'gi', 'G1', 'PI', save=True)
mapDiffContourfLTS_PNG(645, 'hg', 'G1', 'PI', save=True)
mapDiffContourfLTS_PNG(646, 'ip', 'G1', 'PI', save=True)
mapDiffContourfLTS_PNG(647, 'mi', 'G1', 'PI', save=True)
mapDiffContourfLTS_PNG(648, 'no', 'G1', 'PI', save=True)

plt.show()
