# Script to plot multi-model means of low, middle and high cloud fraction changes. 
#
# Since original "multiModelMeanCloudsPredictors", many problems with the preliminary calculations for this for different
# models have been fixed, so I should be able to have as many as 11 models included, not 6. 
# Add more models: BNU r3i1p1; GISS now fixed; redo CCSM4 and CESM for corrected regridding; add MPI, CSIRO and HadGEM2-ES
#
# Doing new script here because I don't want to include seasonal or LTS/EIS calculations.
#
# Rick Russoto
# Started 15 February 2018

import matplotlib.pyplot as plt
import geomipFunctions as gmf
import numpy as np
import netCDF4 as nc4
from mpl_toolkits.basemap import Basemap

modelLats = np.load('npyFiles/modelLats.npy').item()
modelLons = np.load('npyFiles/modelLons.npy').item()  

models = ['bn', 'ca', 'cc', 'ce', 'cs', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']

#Load 2D arrays of low, middle and high cloud fractions for each model, saved by "cloudFractionMaps.py"
#...and take differences between runs
cl_LMH_PI = dict()
cl_LMH_G1 = dict()
cl_LMH_A4 = dict()
cl_low_G1minusPI = dict()
cl_middle_G1minusPI = dict()
cl_high_G1minusPI = dict()
cl_low_A4minusPI = dict()
cl_middle_A4minusPI = dict()
cl_high_A4minusPI = dict()
for key in models:
    print('Cloud fraction loading and difference taking for model: ' + gmf.modelNames[key])
    cl_LMH_PI[key] = np.load('npyFiles/cl_LMH_PI_'+key+'.npy').item()
    cl_LMH_G1[key] = np.load('npyFiles/cl_LMH_G1_'+key+'.npy').item()
    cl_LMH_A4[key] = np.load('npyFiles/cl_LMH_A4_'+key+'.npy').item()    
    
    cl_low_G1minusPI[key] = cl_LMH_G1[key]['low'] - cl_LMH_PI[key]['low']
    cl_middle_G1minusPI[key] = cl_LMH_G1[key]['middle'] - cl_LMH_PI[key]['middle']
    cl_high_G1minusPI[key] = cl_LMH_G1[key]['high'] - cl_LMH_PI[key]['high']
    
    cl_low_A4minusPI[key] = cl_LMH_A4[key]['low'] - cl_LMH_PI[key]['low']
    cl_middle_A4minusPI[key] = cl_LMH_A4[key]['middle'] - cl_LMH_PI[key]['middle']
    cl_high_A4minusPI[key] = cl_LMH_A4[key]['high'] - cl_LMH_PI[key]['high']
    

#Now take multi-model means.
#Use "gmf.collocateModels" function.
#See "multiModelMeanAPRP" for existing example. 

#Lats and lons to interpolate to (1-degree grid)
interpLats = np.linspace(-90., 90., 181)
interpLons = np.linspace(0., 360., 361)


#Sequence of 3 calls to functions: 
#The first regrids them to a common grid, 
#The 2nd takes multi-model mean, 
#And the 3rd calculates where all but 2 models agree on the sign for stippling purposes.

#Doing all the multi-model mean calculations for G1 minus piControl

#Clouds (low)
print('Regridding cloud fractions, low clouds, G1 - piControl: ')
collocatedCloud_low_G1minusPI = gmf.collocateModels(cl_low_G1minusPI, modelLats, modelLons, 
                                                          interpLats, interpLons, method='linear')
mmmCloud_low_G1minusPI = np.mean(collocatedCloud_low_G1minusPI, axis=2)
agreeBut2Cloud_low_G1minusPI = gmf.arraySignAgreement3D(collocatedCloud_low_G1minusPI, 2)

#Clouds (middle)
print('Regridding cloud fractions, middle clouds, G1 - piControl: ')
collocatedCloud_middle_G1minusPI = gmf.collocateModels(cl_middle_G1minusPI, modelLats, modelLons, 
                                                          interpLats, interpLons, method='linear')
mmmCloud_middle_G1minusPI = np.mean(collocatedCloud_middle_G1minusPI, axis=2)
agreeBut2Cloud_middle_G1minusPI = gmf.arraySignAgreement3D(collocatedCloud_middle_G1minusPI, 2)

#Clouds (high)
print('Regridding cloud fractions, high clouds, G1 - piControl: ')
collocatedCloud_high_G1minusPI = gmf.collocateModels(cl_high_G1minusPI, modelLats, modelLons, 
                                                          interpLats, interpLons, method='linear')
mmmCloud_high_G1minusPI = np.mean(collocatedCloud_high_G1minusPI, axis=2)
agreeBut2Cloud_high_G1minusPI = gmf.arraySignAgreement3D(collocatedCloud_high_G1minusPI, 2)


#Same for abrupt4xCO2 - piControl
#Clouds (low)
print('Regridding cloud fractions, low clouds, A4 - piControl: ')
collocatedCloud_low_A4minusPI = gmf.collocateModels(cl_low_A4minusPI, modelLats, modelLons, 
                                                          interpLats, interpLons, method='linear')
mmmCloud_low_A4minusPI = np.mean(collocatedCloud_low_A4minusPI, axis=2)
agreeBut2Cloud_low_A4minusPI = gmf.arraySignAgreement3D(collocatedCloud_low_A4minusPI, 2)

#Clouds (middle)
print('Regridding cloud fractions, middle clouds, A4 - piControl: ')
collocatedCloud_middle_A4minusPI = gmf.collocateModels(cl_middle_A4minusPI, modelLats, modelLons, 
                                                          interpLats, interpLons, method='linear')
mmmCloud_middle_A4minusPI = np.mean(collocatedCloud_middle_A4minusPI, axis=2)
agreeBut2Cloud_middle_A4minusPI = gmf.arraySignAgreement3D(collocatedCloud_middle_A4minusPI, 2)

#Clouds (high)
print('Regridding cloud fractions, high clouds, A4 - piControl: ')
collocatedCloud_high_A4minusPI = gmf.collocateModels(cl_high_A4minusPI, modelLats, modelLons, 
                                                          interpLats, interpLons, method='linear')
mmmCloud_high_A4minusPI = np.mean(collocatedCloud_high_A4minusPI, axis=2)
agreeBut2Cloud_high_A4minusPI = gmf.arraySignAgreement3D(collocatedCloud_high_A4minusPI, 2)




#####   PLOTS   #####


w,h = plt.figaspect(1./2.5)  

#G1 minus piControl plots--using "big text" version used for AGU poster. 
#Low clouds, G1 - piControl
plt.figure(200, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
plotData = mmmCloud_low_G1minusPI*100 #Multiply by 100 to get into percent form
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-5., 5., 21)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('Cloud fraction change (%)', fontsize=24)
cbar.ax.tick_params(labelsize=22)
plt.title('(a) Low clouds (p > 680 hPa)', fontsize=26)   
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agreeBut2Cloud_low_G1minusPI == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
#plt.savefig('plots/multiModelMean_clouds_low_G1minusPI_bigText_v2.png')
plt.savefig('plots/multiModelMean_clouds_low_G1minusPI_bigText_v2_flipped.png')

#Middle clouds, G1 - piControl 
plt.figure(201, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
plotData = mmmCloud_middle_G1minusPI*100 #Multiply by 100 to get into percent form
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-5., 5., 21)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('Cloud fraction change (%)', fontsize=24)
cbar.ax.tick_params(labelsize=22)
plt.title('(b) Middle clouds (440-680 hPa)', fontsize=26)  
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agreeBut2Cloud_middle_G1minusPI == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
#plt.savefig('plots/multiModelMean_clouds_middle_G1minusPI_bigText_v2.png')
plt.savefig('plots/multiModelMean_clouds_middle_G1minusPI_bigText_v2_flipped.png')

#High clouds, G1 - piControl 
plt.figure(202, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
plotData = mmmCloud_high_G1minusPI*100 #Multiply by 100 to get into percent form
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-5., 5., 21)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('Cloud fraction change (%)', fontsize=24)
cbar.ax.tick_params(labelsize=22)
plt.title('(c) High clouds (p < 440 hPa)', fontsize=26)   
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agreeBut2Cloud_high_G1minusPI == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
#plt.savefig('plots/multiModelMean_clouds_high_G1minusPI_bigText_v2.png')
plt.savefig('plots/multiModelMean_clouds_high_G1minusPI_bigText_v2_flipped.png')





#Same for abrupt4xCO2 vs. piControl:
#Low clouds, A4 - piControl
plt.figure(203, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
plotData = mmmCloud_low_A4minusPI*100 #Multiply by 100 to get into percent form
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-10., 10., 21)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('Cloud fraction change (%)', fontsize=24)
cbar.ax.tick_params(labelsize=22)
plt.title('(a) Low clouds (p > 680 hPa)', fontsize=26)   
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agreeBut2Cloud_low_A4minusPI == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
#plt.savefig('plots/multiModelMean_clouds_low_A4minusPI_bigText_v2.png')
plt.savefig('plots/multiModelMean_clouds_low_A4minusPI_bigText_v2_flipped.png')

#Middle clouds, A4 - piControl 
plt.figure(204, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
plotData = mmmCloud_middle_A4minusPI*100 #Multiply by 100 to get into percent form
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-10., 10., 21)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('Cloud fraction change (%)', fontsize=24)
cbar.ax.tick_params(labelsize=22)
plt.title('(b) Middle clouds (440-680 hPa)', fontsize=26)  
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agreeBut2Cloud_middle_A4minusPI == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
#plt.savefig('plots/multiModelMean_clouds_middle_A4minusPI_bigText_v2.png')
plt.savefig('plots/multiModelMean_clouds_middle_A4minusPI_bigText_v2_flipped.png')

#High clouds, A4 - piControl 
plt.figure(205, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
plotData = mmmCloud_high_A4minusPI*100 #Multiply by 100 to get into percent form
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-10., 10., 21)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('Cloud fraction change (%)', fontsize=24)
cbar.ax.tick_params(labelsize=22)
plt.title('(c) High clouds (p < 440 hPa)', fontsize=26)   
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agreeBut2Cloud_high_A4minusPI == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
plt.tight_layout()
#plt.savefig('plots/multiModelMean_clouds_high_A4minusPI_bigText_v2.png')
plt.savefig('plots/multiModelMean_clouds_high_A4minusPI_bigText_v2_flipped.png')



#For the ACP paper, combine everything using the "figurecombiner.tex" script--not using subplots here.


plt.show()