# Counterpart of "multiModelMeanCloudsV2.py" for the LTS/EIS changes. 
#
# Since "multiModelMeanCloudsPredictors" was written, replaced BNU G1 with r3i1p1 realization 
# and added HadGEM2-ES. Didn't add CSIRO or MPI because I don't have "huss" output which I need for EIS.
#
# Rick Russotto
# Started 2018-02-18


import matplotlib.pyplot as plt
import geomipFunctions as gmf
import numpy as np
from mpl_toolkits.basemap import Basemap


modelLats = np.load('npyFiles/modelLats_ta.npy').item()
modelLons = np.load('npyFiles/modelLons_ta.npy').item()  

models = ['bn', 'ca', 'cc', 'ce', 'gi', 'hg', 'ip', 'mi', 'no']


#Load 2D arrays of LTS and EIS for each model, saved by "lowCloudPredictorMaps.py",
#...and take differences between runs
EIS_PI = dict()
EIS_G1 = dict()
EIS_A4 = dict()
LTS_PI = dict()
LTS_G1 = dict()
LTS_A4 = dict()
EIS_G1minusPI = dict()
EIS_A4minusPI = dict()
LTS_G1minusPI = dict()
LTS_A4minusPI = dict()

for key in models:
    print('LTS/EIS loading and difference taking for model: ' + gmf.modelNames[key])
    EIS_PI[key] = np.load('npyFiles/EIS_PI_'+key+'.npy')
    EIS_G1[key] = np.load('npyFiles/EIS_G1_'+key+'.npy')
    EIS_A4[key] = np.load('npyFiles/EIS_A4_'+key+'.npy')
    LTS_PI[key] = np.load('npyFiles/LTS_PI_'+key+'.npy')
    LTS_G1[key] = np.load('npyFiles/LTS_G1_'+key+'.npy')
    LTS_A4[key] = np.load('npyFiles/LTS_A4_'+key+'.npy')
    EIS_G1minusPI[key] = EIS_G1[key] - EIS_PI[key]
    EIS_A4minusPI[key] = EIS_A4[key] - EIS_PI[key]
    LTS_G1minusPI[key] = LTS_G1[key] - LTS_PI[key]
    LTS_A4minusPI[key] = LTS_A4[key] - LTS_PI[key]
    
#Now take multi-model means.
#Use "gmf.collocateModels" function.

#Lats and lons to interpolate to (1-degree grid)
interpLats = np.linspace(-90., 90., 181)
interpLons = np.linspace(0., 360., 361)




#Doing all the multi-model mean calculations for G1 minus piControl

#EIS
print('Regridding EIS, G1 - piControl: ' )
collocatedEIS_G1minusPI = gmf.collocateModels(EIS_G1minusPI, modelLats, modelLons, 
                                                          interpLats, interpLons, method='linear')
mmmEIS_G1minusPI = np.mean(collocatedEIS_G1minusPI, axis=2)
agreeBut2EIS_G1minusPI = gmf.arraySignAgreement3D(collocatedEIS_G1minusPI, 2)

#LTS
print('Regridding LTS, G1 - piControl: ' )
collocatedLTS_G1minusPI = gmf.collocateModels(LTS_G1minusPI, modelLats, modelLons, 
                                                          interpLats, interpLons, method='linear')
mmmLTS_G1minusPI = np.mean(collocatedLTS_G1minusPI, axis=2)
agreeBut2LTS_G1minusPI = gmf.arraySignAgreement3D(collocatedLTS_G1minusPI, 2)



#Same for abrupt4xCO2 minus piControl

#EIS
print('Regridding EIS, A4 - piControl: ' )
collocatedEIS_A4minusPI = gmf.collocateModels(EIS_A4minusPI, modelLats, modelLons, 
                                                          interpLats, interpLons, method='linear')
mmmEIS_A4minusPI = np.mean(collocatedEIS_A4minusPI, axis=2)
agreeBut2EIS_A4minusPI = gmf.arraySignAgreement3D(collocatedEIS_A4minusPI, 2)

#LTS
print('Regridding LTS, A4 - piControl: ' )
collocatedLTS_A4minusPI = gmf.collocateModels(LTS_A4minusPI, modelLats, modelLons, 
                                                          interpLats, interpLons, method='linear')
mmmLTS_A4minusPI = np.mean(collocatedLTS_A4minusPI, axis=2)
agreeBut2LTS_A4minusPI = gmf.arraySignAgreement3D(collocatedLTS_A4minusPI, 2)




#####   PLOTS   #####


w,h = plt.figaspect(1./2.5)  

#G1 minus piControl plots--using "big text" format used for AGU poster. 

#G1 - piControl, EIS
plt.figure(203, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
plotData = mmmEIS_G1minusPI 
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-1.5, 1.5, 16)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('EIS change (K)', fontsize=24)
cbar.ax.tick_params(labelsize=22)
plt.title('(a) Change in Estimated Inversion Strength', fontsize=26)
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agreeBut2EIS_G1minusPI == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
Map.fillcontinents() #Block out land for EIS plot
plt.tight_layout()
#plt.savefig('plots/multiModelMean_EIS_G1minusPI_bigText_v2.png')
plt.savefig('plots/multiModelMean_EIS_G1minusPI_bigText_v2_flipped.png')

#G1 - piControl, LTS
plt.figure(204, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
plotData = mmmLTS_G1minusPI 
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-1.5, 1.5, 16)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('LTS change (K)', fontsize=24)
cbar.ax.tick_params(labelsize=22)
plt.title('(b) Change in Lower Tropospheric Stability', fontsize=26)
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agreeBut2LTS_G1minusPI == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
Map.fillcontinents() #Block out land for LTS plot
plt.tight_layout()
#plt.savefig('plots/multiModelMean_LTS_G1minusPI_bigText_v2.png')
plt.savefig('plots/multiModelMean_LTS_G1minusPI_bigText_v2_flipped.png')


#A4 - piControl, EIS
plt.figure(303, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
plotData = mmmEIS_A4minusPI 
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-8.0, 8.0, 16)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('EIS change (K)', fontsize=24)
cbar.ax.tick_params(labelsize=22)
plt.title('(a) Change in Estimated Inversion Strength', fontsize=26)
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agreeBut2EIS_A4minusPI == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
Map.fillcontinents() #Block out land for EIS plot
plt.tight_layout()
#plt.savefig('plots/multiModelMean_EIS_A4minusPI_bigText_v2.png')
plt.savefig('plots/multiModelMean_EIS_A4minusPI_bigText_v2_flipped.png')

#A4 - piControl, LTS
plt.figure(304, figsize=(w,h))
Map = Basemap(projection='moll', resolution='l', lon_0=0, lat_0=0)
Map.drawcoastlines(linewidth=1)
Map.drawparallels(np.linspace(-90.,90.,7))
Map.drawmeridians(np.linspace(-150., 150., 11))
lon = interpLons
lat = interpLats    
lons, lats = np.meshgrid(lon, lat)
plotData = mmmLTS_A4minusPI 
cmap = plt.cm.RdBu_r  
clevs=np.linspace(-8.0, 8.0, 16)
Map.contourf(lons, lats, plotData, latlon=True, cmap=cmap, levels=clevs, extend='both')
cbar = plt.colorbar(ticks=np.linspace(min(clevs), max(clevs), 11))  
cbar.ax.set_ylabel('LTS change (K)', fontsize=24)
cbar.ax.tick_params(labelsize=22)
plt.title('(b) Change in Lower Tropospheric Stability', fontsize=26)
#Add hatching (next 2 lines)
zm = np.ma.masked_where(agreeBut2LTS_A4minusPI == True, plotData)
Map.pcolor(lons, lats, zm, latlon=True, cmap='BrBG', alpha=0, hatch='xx') 
Map.fillcontinents() #Block out land for LTS plot
plt.tight_layout()
#plt.savefig('plots/multiModelMean_LTS_A4minusPI_bigText_v2.png')
plt.savefig('plots/multiModelMean_LTS_A4minusPI_bigText_v2_flipped.png')


#For the ACP paper, combine everything using the "figurecombiner.tex" script--not using subplots here.


plt.show()

