# Script to load latitudes and longitudes for each model and save as npy file, so that I don't have to 
# load actual NetCDF data files in scripts where I don't need the data. 
# Get these from the "tas" data files. (11 models--excluding HadCM3 and EC-Earth)
# 
# Rick Russotto
# Started 2018-01-09

import netCDF4 as nc4
import numpy as np

models = ['bn', 'ca', 'cc', 'ce', 'cs', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']

#Paths to the files
fpaths_PI = dict()
fpaths_PI['bn'] = '../nobackup/tas_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc'
fpaths_PI['ca'] = '../nobackup/tas_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc'
fpaths_PI['cc'] = '../nobackup/tas_Amon_CCSM4_piControl_r1i1p1_080001-130012.nc'
fpaths_PI['ce'] = '../nobackup/tas_Amon_CESM-CAM5.1-FV_piControl_r1i1p1_000101-005012.nc'
fpaths_PI['cs'] = '../nobackup/tas_Amon_CSIRO-Mk3L-1-2_piControl_r1i1p1_000101-100012.nc'
fpaths_PI['gi'] = '../nobackup/tas_Amon_GISS-E2-R_piControl_r1i1p1_185001-191912.nc'
fpaths_PI['hg'] = '../nobackup/tas_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc'
fpaths_PI['ip'] = '../nobackup/tas_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-279912.nc'
fpaths_PI['mi'] = '../nobackup/tas_Amon_MIROC-ESM_piControl_r1i1p1_180001-233012.nc'
fpaths_PI['mp'] = '../nobackup/tas_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-203512.nc'
fpaths_PI['no'] = '../nobackup/tas_Amon_NorESM1-M_piControl_r1i1p1_070001-115012.nc'

Datasets_PI = dict()
for key in models:
    Datasets_PI[key] = nc4.Dataset(fpaths_PI[key])
    
modelLats = dict()
modelLons = dict()
for key in models:
    modelLats[key] = Datasets_PI[key].variables['lat'][:]
    modelLons[key] = Datasets_PI[key].variables['lon'][:]
    
    
#Save dicts as npy files
np.save('npyFiles/modelLats.npy', modelLats)
np.save('npyFiles/modelLons.npy', modelLons)
    
    
    
#For the HadGEM2-ES model, the "ta" lat-lon grid was different from that for "tas". 
#Need this version for the radiative kernel stuff. So, save an alternate version of the grids:
    
Dataset_PI_hg_ta = nc4.Dataset('../nobackup/ta_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc')
modelLats_ta = dict(modelLats)
modelLons_ta = dict(modelLons)
modelLats_ta['hg'] = Dataset_PI_hg_ta.variables['lat'][:]
modelLons_ta['hg'] = Dataset_PI_hg_ta.variables['lon'][:]

np.save('npyFiles/modelLats_ta.npy', modelLats_ta)
np.save('npyFiles/modelLons_ta.npy', modelLons_ta)