# Script to plot vertical profiles of temperature ("ta") in different
# GeoMIP models. 
#
# Rick Russotto
# Started 17 June 2016


import numpy as np
import matplotlib.pyplot as plt
import netCDF4 as nc4
import geomipFunctions as gmf
import os
from collections import OrderedDict



#Paths for the files
folder = '../nobackup/' 
#Temperatures
fpaths_ta_G1 = OrderedDict()
#fpaths_ta_G1['bn'] = folder+'ta_Amon_BNU-ESM_G1_r1i1p1_185001-190012.nc' 
fpaths_ta_G1['bn'] = folder+'ta_Amon_BNU-ESM_G1_r3i1p1_185001-194912.nc'
fpaths_ta_G1['ca'] = folder+'ta_Amon_CanESM2_G1_r1i1p1_185001-194912.nc'
fpaths_ta_G1['cc'] = folder+'ta_Amon_CCSM4_G1_r1i1p1_185001-192412.nc'   
fpaths_ta_G1['ce'] = folder+'ta_Amon_CESM1-CAM5-1-FV2_G1_r1i1p1_000101-006001.nc'
fpaths_ta_G1['cs'] = folder+'ta_Amon_CSIRO-Mk3L-1-2_G1_r1i1p1_000101-007012.nc'
fpaths_ta_G1['gi'] = folder+'ta_Amon_GISS-E2-R_G1_r1i1p1_185001-191912.nc'
fpaths_ta_G1['hg'] = folder+'ta_Amon_HadGEM2-ES_G1_r1i1p1_185912-190911.nc' #Skip for now
fpaths_ta_G1['ip'] = folder+'ta_Amon_IPSL-CM5A-LR_G1_r1i1p1_185001-189912.nc'
fpaths_ta_G1['mi'] = folder+'ta_Amon_MIROC-ESM_G1_r1i1p1_000101-005212.nc'
fpaths_ta_G1['mp'] = folder+'ta_Amon_MPI-ESM-LR_G1_r2i1p1_185001-191912.nc'
fpaths_ta_G1['no'] = folder+'ta_Amon_NorESM1-M_G1_r1i1p1_000101-005012.nc'


fpaths_ta_PI = OrderedDict()
fpaths_ta_PI['bn'] = folder+'ta_Amon_BNU-ESM_piControl_r1i1p1_145001-200812.nc'
fpaths_ta_PI['ca'] = folder+'ta_Amon_CanESM2_piControl_r1i1p1_201501-231012.nc'
fpaths_ta_PI['cc'] = folder+'ta_Amon_CCSM4_piControl_r1i1p1_080001-084912.nc'
fpaths_ta_PI['ce'] = folder+'ta_Amon_CESM1-CAM5-1-FV2_piControl_r1i1p1_000101-005012.nc'
fpaths_ta_PI['cs'] = folder+'ta_Amon_CSIRO-Mk3L-1-2_piControl_r1i1p1_000101-010012.nc'
fpaths_ta_PI['gi'] = folder+'ta_Amon_GISS-E2-R_piControl_r1i1p1_185001-191912.nc'
fpaths_ta_PI['hg'] = folder+'ta_Amon_HadGEM2-ES_piControl_r1i1p1_185912-190911.nc'
fpaths_ta_PI['ip'] = folder+'ta_Amon_IPSL-CM5A-LR_piControl_r1i1p1_180001-189912.nc'
fpaths_ta_PI['mi'] = folder+'ta_Amon_MIROC-ESM_piControl_r1i1p1_180001-193912.nc'
fpaths_ta_PI['mp'] = folder+'ta_Amon_MPI-ESM-LR_piControl_r1i1p1_185001-189912.nc'
fpaths_ta_PI['no'] = folder+'ta_Amon_NorESM1-M_piControl_r1i1p1_070101-079912.nc'


fpaths_ta_A4 = OrderedDict()
fpaths_ta_A4['bn'] = folder+'ta_Amon_BNU-ESM_abrupt4xCO2_r1i1p1_185001-199912.nc'
fpaths_ta_A4['ca'] = folder+'ta_Amon_CanESM2_abrupt4xCO2_r1i1p1_185001-199912.nc'
fpaths_ta_A4['cc'] = folder+'ta_Amon_CCSM4_abrupt4xCO2_r1i1p1_185001-189912.nc'
fpaths_ta_A4['ce'] = folder+'ta_Amon_CESM1-CAM5-1-FV2_abrupt4xCO2_r3i1p1_000101-015012.nc'
fpaths_ta_A4['cs'] = folder+'ta_Amon_CSIRO-Mk3L-1-2_abrupt4xCO2_r1i1p1_000101-010012.nc'
fpaths_ta_A4['gi'] = folder+'ta_Amon_GISS-E2-R_abrupt4xCO2_r1i1p1_185001-191912.nc'
fpaths_ta_A4['hg'] = folder+'ta_Amon_HadGEM2-ES_abrupt4xCO2_r1i1p1_185912-191011.nc'
fpaths_ta_A4['ip'] = folder+'ta_Amon_IPSL-CM5A-LR_abrupt4xCO2_r1i1p1_185001-194912.nc'
fpaths_ta_A4['mi'] = folder+'ta_Amon_MIROC-ESM_abrupt4xCO2_r1i1p1_000101-014012.nc'
fpaths_ta_A4['mp'] = folder+'ta_Amon_MPI-ESM-LR_abrput4xCO2_r1i1p1_185001-189912.nc'
fpaths_ta_A4['no'] = folder+'ta_Amon_NorESM1-M_abrupt4xCO2_r1i1p1_000101-010012.nc'




#Wheter to load from a file (so I don't have to take zonal mean multiple times)
loadNpyFile_ta_G1 = OrderedDict()
loadNpyFile_ta_G1['bn'] = False
loadNpyFile_ta_G1['ca'] = False
loadNpyFile_ta_G1['cc'] = False
loadNpyFile_ta_G1['ce'] = False
loadNpyFile_ta_G1['cs'] = False
loadNpyFile_ta_G1['gi'] = False
loadNpyFile_ta_G1['hg'] = False
loadNpyFile_ta_G1['ip'] = False
loadNpyFile_ta_G1['mi'] = False #Reported more levels than required (intermediate)
loadNpyFile_ta_G1['mp'] = False
loadNpyFile_ta_G1['no'] = False

loadNpyFile_ta_PI = OrderedDict()
loadNpyFile_ta_PI['bn'] = False
loadNpyFile_ta_PI['ca'] = False
loadNpyFile_ta_PI['cc'] = False
loadNpyFile_ta_PI['ce'] = False
loadNpyFile_ta_PI['cs'] = False
loadNpyFile_ta_PI['gi'] = False
loadNpyFile_ta_PI['hg'] = False
loadNpyFile_ta_PI['ip'] = False
loadNpyFile_ta_PI['mi'] = False
loadNpyFile_ta_PI['mp'] = False
loadNpyFile_ta_PI['no'] = False

loadNpyFile_ta_A4 = OrderedDict()
loadNpyFile_ta_A4['bn'] = False
loadNpyFile_ta_A4['ca'] = False
loadNpyFile_ta_A4['cc'] = False
loadNpyFile_ta_A4['ce'] = False
loadNpyFile_ta_A4['cs'] = False
loadNpyFile_ta_A4['gi'] = False
loadNpyFile_ta_A4['hg'] = False
loadNpyFile_ta_A4['ip'] = False
loadNpyFile_ta_A4['mi'] = False
loadNpyFile_ta_A4['mp'] = False
loadNpyFile_ta_A4['no'] = False


#Load NetCDF files & create NetCDF Dataset objects
Datasets_ta_G1 = OrderedDict()
for key, value in fpaths_ta_G1.items():
    print('Loading NetCDF file (ta): G1: ' + gmf.modelNames[key]) #Debug
    Datasets_ta_G1[key] = nc4.Dataset(value)
    
Datasets_ta_PI = OrderedDict()    
for key, value in fpaths_ta_PI.items():
    print('Loading NetCDF file (ta): piControl: ' + gmf.modelNames[key]) #Debug
    Datasets_ta_PI[key] = nc4.Dataset(value)    

Datasets_ta_A4 = OrderedDict()    
for key, value in fpaths_ta_A4.items():
    print('Loading NetCDF file (ta): abrupt4xCO2: ' + gmf.modelNames[key]) #Debug
    Datasets_ta_A4[key] = nc4.Dataset(value)    



#Calculate zonal means, averaged over 40 years
#For G1 and Abrupt4xCO2, the average was over years 11-50 in the Kravitz et al. papers; (10-49 in Python indexing)
#For piControl, it doesn't matter which 40 years, but use 1st 40 years for consistency.


#The standard pressure levels from CMIP5: (hPa)     
levs = np.array([1000, 925, 850, 700, 600, 500, 400, 300, 250, 200, 150, 100, 70, 50, 30, 20, 10]) 
npyFolder = 'npyFiles/' #Locations of the npy files


#Do the pressure interpolation and zonal mean calculation and save to npy file, or load from existing npy files
#G1: years 10-49 (if year indices start from 0)
zonalMeans_ta_G1 = OrderedDict()
for key, value in Datasets_ta_G1.items():
    if not(loadNpyFile_ta_G1[key]):
        print('Calculating zonal mean temperature: G1: ' + gmf.modelNames[key])
        zonalMeans_ta_G1[key] = gmf.zonalMeanMultiYearPressureCoordinates(value, 'ta', 10, 49)
        #Subset to the pressure levels of interest
        if key == 'mi': #This model reported extra layers in between the standard CESM ones--listed at bottom
            zonalMeans_ta_G1[key] = np.concatenate((zonalMeans_ta_G1[key][0:10,:],
                                                    zonalMeans_ta_G1[key][11:12,:],
                                                    zonalMeans_ta_G1[key][14:15,:],
                                                    zonalMeans_ta_G1[key][17:21,:],
                                                    zonalMeans_ta_G1[key][22:23,:]),    axis=0)
        else:
            zonalMeans_ta_G1[key] = zonalMeans_ta_G1[key][0:17,:] #Get rid of any pressure levels beyond 10 hPa
        if not(key in ['cc', 'cs', 'gi', 'hg', 'ip', 'mi', 'ce']): #These models have masked arrays which...
            np.save(npyFolder+'zonalMeans_ta_G1_'+key+'.npy', zonalMeans_ta_G1[key]) #...can't be saved using np.save
        else:
            zonalMeans_ta_G1[key].dump(npyFolder+'zonalMeans_ta_G1_'+key+'.npy')
    else:
        print('Loading zonal mean temperature from npy file: G1: ' + gmf.modelNames[key])
        zonalMeans_ta_G1[key] = np.load(npyFolder+'zonalMeans_ta_G1_'+key+'.npy')
        
#piControl: years 0-39 (doesn't matter which 40 years in this case)        
zonalMeans_ta_PI = OrderedDict()
for key, value in Datasets_ta_PI.items():
    if not(loadNpyFile_ta_PI[key]):
        print('Calculating zonal mean temperature: piControl: ' + gmf.modelNames[key])
        zonalMeans_ta_PI[key] = gmf.zonalMeanMultiYearPressureCoordinates(value, 'ta', 0, 39)        
        #Subset to the pressure levels of interest
        if key == 'mi': #This model reported extra layers in between the standard CESM ones--listed at bottom
            zonalMeans_ta_PI[key] = np.concatenate((zonalMeans_ta_PI[key][0:10,:],
                                                    zonalMeans_ta_PI[key][11:12,:],
                                                    zonalMeans_ta_PI[key][14:15,:],
                                                    zonalMeans_ta_PI[key][17:21,:],
                                                    zonalMeans_ta_PI[key][22:23,:]),    axis=0)
        else:
            zonalMeans_ta_PI[key] = zonalMeans_ta_PI[key][0:17,:] #Get rid of any pressure levels beyond 10 hPa
        if not(key in ['cc', 'cs', 'gi', 'hg', 'ip', 'mi', 'ce']): #Maksed arrays--use dump
            np.save(npyFolder+'zonalMeans_ta_PI_'+key+'.npy', zonalMeans_ta_PI[key])
        else:
            zonalMeans_ta_PI[key].dump(npyFolder+'zonalMeans_ta_PI_'+key+'.npy')
    else:
        print('Loading zonal mean temperature from npy file: piControl: ' + gmf.modelNames[key])
        zonalMeans_ta_PI[key] = np.load(npyFolder+'zonalMeans_ta_PI_'+key+'.npy')        
        
#abrupt4xCO2: years 10-49        
zonalMeans_ta_A4 = OrderedDict()
for key, value in Datasets_ta_A4.items():
    if not(loadNpyFile_ta_A4[key]):
        print('Calculating zonal mean temperature: abrupt4xCO2: ' + gmf.modelNames[key])
        zonalMeans_ta_A4[key] = gmf.zonalMeanMultiYearPressureCoordinates(value, 'ta', 10, 49)
        #Subset to the pressure levels of interest
        if key == 'mi': #This model reported extra layers in between the standard CESM ones--listed at bottom
            zonalMeans_ta_A4[key] = np.concatenate((zonalMeans_ta_A4[key][0:10,:],
                                                    zonalMeans_ta_A4[key][11:12,:],
                                                    zonalMeans_ta_A4[key][14:15,:],
                                                    zonalMeans_ta_A4[key][17:21,:],
                                                    zonalMeans_ta_A4[key][22:23,:]),    axis=0)
        else:
            zonalMeans_ta_A4[key] = zonalMeans_ta_A4[key][0:17,:] #Get rid of any pressure levels beyond 10 hPa
        if not(key in ['cc', 'cs', 'gi', 'hg', 'ip', 'mi', 'ce']): #Masked arrays--use dump
            np.save(npyFolder+'zonalMeans_ta_A4_'+key+'.npy', zonalMeans_ta_A4[key])
        else:
            zonalMeans_ta_A4[key].dump(npyFolder+'zonalMeans_ta_A4_'+key+'.npy')
    else:
        print('Loading zonal mean temperature from npy file: abrupt4xCO2: ' + gmf.modelNames[key])
        zonalMeans_ta_A4[key] = np.load(npyFolder+'zonalMeans_ta_A4_'+key+'.npy')        

print('Finished zonal mean calculations or loading.')






#####   PLOTS   #####


#For ACP paper: subplot version showing 11 models, with legend in 12th slot

keys = ['bn', 'ca', 'cc', 'ce', 'cs', 'gi', 'hg', 'ip', 'mi', 'mp', 'no']
(w,h) = plt.figaspect(0.7)*1.9
fig = plt.figure(500, figsize=(w,h))
for i in range(11):
    plt.subplot(3,4,i+1)
    lat = Datasets_ta_G1[keys[i]].variables['lat'][:]
    plotData = zonalMeans_ta_G1[keys[i]] - zonalMeans_ta_PI[keys[i]]
    thePlot = plt.contourf(np.sin(lat*np.pi/180.), levs, plotData, levels=np.linspace(-3.,3.,12), cmap=plt.cm.RdBu_r, extend='both')
    plt.gca().invert_yaxis()
    if i in range(8,11):
        plt.xlabel('Latitude ($^\circ$N)', fontsize=15)
    plt.xticks(np.sin(np.array([-90, -30, 0, 30, 90])*np.pi/180.),  ['-90', '-30', '0', '30', '90'])
    if not(np.mod(i,4)):
        plt.ylabel('Pressure (hPa)', fontsize=15) #Y axis labels running alon the left side
    plt.yticks([1000, 800, 600, 400, 200, 0])
    plt.title(gmf.modelNames[keys[i]], fontsize=15)
    plt.gca().set_axis_bgcolor('k') 
#Colorbar
cbar = plt.colorbar(thePlot, cax=fig.add_axes([0.78, 0.18, 0.2, 0.03]), ticks=np.linspace(-3, 3, 7), orientation='horizontal')
cbar.ax.set_xlabel('Temperature change (K)', fontsize=15)
cbar.ax.tick_params(labelsize=15)
plt.tight_layout()
plt.savefig('plots/ta_zonalMean_diff_ACP_paperVersion.pdf')



plt.show()


#MIROC levels (Pa): 
#
#plev = 100000, 92500, 85000, 70000, 60000, 50000, 40000, 30000, 25000, 
#   20000, 17000, 15000, 13000, 11500, 10000, 9000, 8000, 7000, 5000, 3000, 
#   2000, 1500, 1000, 700, 500, 300, 200, 150, 100, 50, 30.0000011920929, 
#   20.0000002980232, 10.0000001490116, 5.00000007450581, 2.99999993294477 ;
#
#CMIP5 levels (hPa):
#
#levs = np.array([1000, 925, 850, 700, 600, 500, 400, 300, 250, 200, 150, 100, 70, 50, 30, 20, 10]) 
#
#So I need indices: 0,1,2,3,4,5,6,7,8,9,11,14,17,18,19,20,22
# or, 0:9,11,14,17:20,22 (but remember to add 1 to upper bound for Python indexing)

